/*
 * Created on Mar 5, 2005
 */
package org.exoplatform.faces.cms;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIDateInput;
import org.exoplatform.faces.core.component.UIHtmlTextArea;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.services.idgenerator.IDGeneratorService;

/**
 * @author benjaminmestrallet
 */
public class DialogUtil {
  
  private static final String TEXT = "text";
  private static final String TEXTAREA = "textarea";
  private static final String HTMLTEXTAREA = "htmltextarea";
  private static final String DATE = "date";
  private static final String UPLOAD = "upload";
  
  public static List getUIInputs(Node dialog) throws Exception {            
    List inputs = new ArrayList();
    Node node = dialog.getNode("node");
    buildRecursively(node, inputs);
    return inputs;
  }
  
  public static void buildRecursively(Node currentNode, List inputs) throws Exception{
    PropertyIterator propertyIterator = currentNode.getProperties();
    String component = "";
    String label = "";
    while (propertyIterator.hasNext()) {
      Property property = propertyIterator.nextProperty();
      String name = property.getName();
      if("component".equals(name))
        component = property.getString();
      else if("label".equals(name))
        label = property.getString();
    }
    IDGeneratorService idGenerator = (IDGeneratorService)PortalContainer.getComponent(
        IDGeneratorService.class);
    String id = "a" + idGenerator.generateStringID(currentNode);
    InputInfo input = new InputInfo(id, currentNode.getPath(), label, 
        getComponent(id, component), false); 
    inputs.add(input);
    NodeIterator nodeIterator = currentNode.getNodes();
    while (nodeIterator.hasNext()) {
      Node node = (Node) nodeIterator.next();
      buildRecursively(node, inputs);      
    }
  }
  
  private static UIInput getComponent(String key, String component) {    
    if (component == null || TEXT.equals(component)) {
      return new UIStringInput(key, "");        
    } else if (TEXTAREA.equals(component)) {
      return new UITextArea(key, "");                
    } else if (HTMLTEXTAREA.equals(component)) {
      return new UIHtmlTextArea(key, "", "100%", "300px");           
    } else if (DATE.equals(component)) {
      return new UIDateInput(key, new Date());
    } else if (UPLOAD.equals(component)) {
      return new UIUploadInput(key);   
    }
    return null;
  }
  
}
