package org.exoplatform.portlets.content.actions.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.jcr.PropertyType;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.actions.ActionPlugin;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitionValue;

public class UIActionTypeForm extends UISimpleMulltipartForm {

  protected UISelectBox typeInput_;
  protected ActionServiceContainer actionsServiceContainer_;
  protected UISelectBox actionExecutablesInput_;
  protected String actionTypeName_;
  protected UIStringInput variablesInput_;
  protected UIStringInput nameInput_;
  protected RepositoryService repositoryService_;
  protected UICheckBox moveInput_;

  public UIActionTypeForm(RepositoryService repositoryService,
      ActionServiceContainer actionServiceContainer) throws Exception {
    super("actionTypeForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    actionsServiceContainer_ = actionServiceContainer;
    repositoryService_ = repositoryService;

    nameInput_ = new UIStringInput("name", "");
    add(nameInput_);
    
    moveInput_ = new UICheckBox("isMove", null);
    add(moveInput_);

    List options = new ArrayList();
    actionExecutablesInput_ = new UISelectBox("actionExecutables", "", options);
    add(actionExecutablesInput_);

    variablesInput_ = new UIStringInput("variables", "");
    add(variablesInput_);

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public String getActionExecutableLabel() {
    ActionPlugin actionPlugin = actionsServiceContainer_
        .getActionPluginForActionType(actionTypeName_);
    return actionPlugin.getActionExecutableLabel();
  }

  public void createNewActionType(String actionTypeName) throws Exception {
    setActionTypeName(actionTypeName);
    reset();
    nameInput_.setEditable(true);
  }

  public void editActionType(String actionTypeName) throws Exception {
    setActionTypeName(actionTypeName);
    reset();
    nameInput_.setEditable(false);
    nameInput_.setValue(actionTypeName);
    ActionPlugin actionPlugin = actionsServiceContainer_
        .getActionPluginForActionType(actionTypeName);
    String exec = actionPlugin.getActionExecutable(actionTypeName);
    actionExecutablesInput_.setValue(exec);
    Collection variables = actionPlugin.getVariableNames(actionTypeName);
    int i = 0;
    for (Iterator iter = variables.iterator(); iter.hasNext(); i++) {
      String variable = (String) iter.next();
      if (i == 0)
        variablesInput_.setValue(variable);
      else
        addValue(new UIStringInput("variables", variable));
    }
  }

  public void setActionTypeName(String actionTypeName) throws Exception {
    actionTypeName_ = actionTypeName;
    List options = new ArrayList();
    ActionPlugin actionPlugin = actionsServiceContainer_
        .getActionPluginForActionType(actionTypeName);
    Collection<String> actionExecutables = actionPlugin.getActionExecutables();
    for (Iterator iter = actionExecutables.iterator(); iter.hasNext();) {
      String actionExec = (String) iter.next();
      options.add(new SelectItem(actionExec, actionExec));
    }
    actionExecutablesInput_.setOptions(options);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionTypeForm uiForm = (UIActionTypeForm) event.getSource();

      Information iprovider = findInformationProvider(uiForm);
      String childActionType = uiForm.nameInput_.getValue();
      if (!childActionType.startsWith("exo:")) {
        iprovider.addMessage(new ExoFacesMessage(
            "#{UIActionTypeForm.msg.action-type-not-added}"));
        return;
      }      
      
      List<String> variables = new ArrayList<String>();
      String varName = uiForm.variablesInput_.getName();
      if (uiForm.isMultiValue(varName)) {
        List<UIInput> values = uiForm.getMultipleValuesUIInput(varName);
        for (Iterator iter = values.iterator(); iter.hasNext();) {
          UIInput valueInput = (UIInput) iter.next();
          variables.add(valueInput.getName());
        }
      } else {
        variables.add(uiForm.variablesInput_.getName());
      }

      try {
        uiForm.actionsServiceContainer_.createActionType(childActionType, uiForm.actionTypeName_, 
            uiForm.actionExecutablesInput_.getValue(), variables, uiForm.moveInput_.getSelect());        
        iprovider.addMessage(new ExoFacesMessage(
            "#{UIActionTypeForm.msg.action-type-added}"));
        ((UIExoComponent) uiForm.getParent())
            .setRenderedSibling(UIActionTypeList.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionTypeForm uiForm = (UIActionTypeForm) event.getSource();
      ((UIExoComponent) uiForm.getParent())
          .setRenderedSibling(UIActionTypeList.class);
    }
  }

}
