/*
 * Created on Apr 13, 2005
 */
package org.exoplatform.portlets.content.display.dynamic.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.portlets.content.display.model.ContentConfigManipulationsUtils;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UIContentConfigForm extends UISimpleForm{

  private UIStringInput nameInput_ ;
  private UISelectBox documentTypesInput_;
  private ContentConfig config_ ;
  private boolean isNew;
  private TemplateService templateService_;
  
  public UIContentConfigForm(TemplateService templatesService) throws Exception{
    super("configForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;
    
    templateService_ = templatesService;
    
    nameInput_ = new UIStringInput("name", "") ;
    add(nameInput_);        
    
		documentTypesInput_ = new UISelectBox("documentType", "", null);    
    add(documentTypesInput_);    
    
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
  
  public void initOptions() throws Exception{
    List options = new ArrayList();   
    NodeIterator documentTypes = templateService_.getDocumentTypes();
    while(documentTypes.hasNext()){
      Node node = documentTypes.nextNode();
      options.add(new SelectItem(node.getName(), node.getName()));
    }
    documentTypesInput_.setOptions(options);        
  }
  
  public boolean isNew() {
    return isNew;
  }
  
  public void setContentConfig(ContentConfig contentConfig) {
    this.config_ = contentConfig;
    if(config_ != null) {
      isNew = false;
      nameInput_.setValue(config_.getName());
      nameInput_.setEditable(false);
    } else {
      isNew = true;
      nameInput_.setValue("");
      nameInput_.setEditable(true);
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfigForm uiForm = (UIContentConfigForm) event.getComponent() ;
      if(uiForm.config_ == null) {
        uiForm.config_ = new ContentConfig() ;
        uiForm.config_.setConfigType(ContentConfigConstants.ENTRY_TYPE);
      }
      String name = uiForm.nameInput_.getValue() ;
      if(name == null || name.length() == 0) {
        Information iprovider = findInformationProvider(uiForm) ;
        iprovider.addMessage(new ExoFacesMessage("#{UIContentConfigForm.msg.name-is-null}")) ;
        return ;
      }
      uiForm.config_.setName(name);
      
      if(uiForm.isNew()){
        String documentType = uiForm.documentTypesInput_.getValue();
        uiForm.config_.setDocumentType(documentType);        
        String uri = ContentUtil.getPortalUri() + "/" + name;
        uiForm.config_.setUri(uri);
        UIContentEditor uiEditor = (UIContentEditor)uiForm.getSibling(UIContentEditor.class) ;
        uiEditor.setContentConfig(uiForm.config_);
        uiEditor.setIsNew(true);
        uiForm.setRenderedSibling(UIContentEditor.class) ;        
      } else {
        UIContentConfig uiConfig = (UIContentConfig)uiForm.getSibling(UIContentConfig.class) ;
        ContentConfigManipulationsUtils.saveContentConfig(uiForm.config_);
        Session session = ((UIAdminContent)uiForm.getAncestorOfType(
            UIAdminContent.class)).getSession();         
        uiConfig.updateConfigs(session);
        uiForm.setRenderedSibling(UIContentConfig.class) ;        
      }
    }
  }  
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfigForm uiForm = (UIContentConfigForm) event.getComponent() ;
      uiForm.setRenderedSibling(UIContentConfig.class) ;
    }
  }
  
}
