package org.exoplatform.portlets.content.display.dynamic.component;

import java.util.GregorianCalendar;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Session;

import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.portlets.content.display.model.ContentConfigManipulationsUtils;
import org.exoplatform.services.cms.CmsConfigurationService;

public class UIReferenceForm extends UISimpleForm implements SelectorHandler{
  
  public static final String SEARCH_NODE = "searchNode";  
  public static final String REFERENCE_INPUT = "reference";  
  
  private UIStringInput nameInput_ ;
  private UIStringInput referenceInput_ ;
  private ContentConfig config_;
  private boolean isNew;
  private CmsConfigurationService cmsConf;

  public UIReferenceForm(CmsConfigurationService cmsConf) {
    super("referenceForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;    
    this.cmsConf = cmsConf;
    
    nameInput_ = new UIStringInput("name", "") ;
    add(nameInput_);

    referenceInput_ = new UIStringInput(REFERENCE_INPUT, "") ;
    add(referenceInput_);    
    
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SearchNodeActionListener.class, SEARCH_NODE) ;
  }
  
  public boolean isNew(){
    return isNew;
  }

  public void setContentConfig(ContentConfig contentConfig) throws Exception{
    this.config_ = contentConfig;
    if(config_ != null) {
      isNew = false;
      nameInput_.setValue(config_.getName());
      nameInput_.setEditable(false);
      Session session = ((UIAdminContent) getAncestorOfType(
          UIAdminContent.class)).getSession();
      String path = ContentUtil.resolveLocalisedViewPath(contentConfig.getUri());
      Node viewNode = (Node) session.getItem(path);
      Node contentNode = (Node) viewNode.getPrimaryItem();
      String reference = contentNode.getProperty("jcr:data").getString();
      referenceInput_.setValue(reference);
    } else {
      isNew = true;
      nameInput_.setValue("");
      nameInput_.setEditable(true);
      referenceInput_.setValue("");
    }
  }
  
  public String getSearchNodeAction() { return SEARCH_NODE; }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIReferenceForm.class);
  }

  public void select(Selector selector) throws Exception {
    String selectedNodePath = selector.getInputValue();
    UIStringInput input = (UIStringInput) getUIInput(REFERENCE_INPUT) ;
    input.setValue(selectedNodePath);
    selector.setRenderedSibling(UIReferenceForm.class);
  }  
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIReferenceForm uiForm = (UIReferenceForm) event.getComponent() ;
      if(uiForm.config_ == null) {
        uiForm.config_ = new ContentConfig() ;
        uiForm.config_.setConfigType(ContentConfigConstants.REFERENCE_TYPE);
      }
      String name = uiForm.nameInput_.getValue() ;
      if(name == null || name.length() == 0) {
        Information iprovider = findInformationProvider(uiForm) ;
        iprovider.addMessage(new ExoFacesMessage("#{UIReferenceForm.msg.name-is-null}")) ;
        return ;
      }
      uiForm.config_.setName(name);
      String uri = ContentUtil.getPortalUri() + "/" + name;
      
      uiForm.config_.setUri(uri);               
      
      String reference = uiForm.referenceInput_.getValue();
      if(reference == null || reference.length() == 0) {        
        reference = uri;
      }
      if(uiForm.isNew) {
        int pos = ContentConfigManipulationsUtils.getNewPosition();
        uiForm.config_.setPosition("" + pos);
      }      
      ContentConfigManipulationsUtils.saveContentConfig(uiForm.config_);
      
      Session session = ((UIAdminContent)uiForm.getAncestorOfType(UIAdminContent.class)).getSession();  

      String sitePath = ContentUtil.getLocalisedAbsoluteSitePath();
      Node homeNode = null;
      try {
        homeNode = (Node) session.getItem(sitePath);
      } catch (PathNotFoundException e) {
        return;
      }    

      String parentPath = uri.substring(0, uri.lastIndexOf("/"));
      String fileName = uri.substring(uri.lastIndexOf("/") + 1);
      
      Node node = ContentUtil.createNodeFromNode(homeNode, parentPath);
      node = node.addNode(fileName, "nt:file");      
            
      try {
        node = node.getNode("jcr:content");
      } catch (PathNotFoundException e) {
        node = node.addNode("jcr:content", "nt:resource");
      }      
      node.setProperty("jcr:encoding", "UTF-8");
      node.setProperty("jcr:mimeType", "text/html");
      node.setProperty("jcr:lastModified", new GregorianCalendar());    
      node.setProperty("jcr:data", reference);
      homeNode.save();            
      
      UIContentConfig uiConfig = (UIContentConfig)uiForm.getSibling(UIContentConfig.class) ;
      uiConfig.updateConfigs(session);
      uiForm.setRenderedSibling(UIContentConfig.class) ;        
    }
  }    
  
  static public class SearchNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIReferenceForm uiForm = (UIReferenceForm) event.getComponent();
      Session session = ((UIAdminContent)uiForm.getAncestorOfType(UIAdminContent.class)).getSession();
      JcrBrowser jcrBrowser = (JcrBrowser) uiForm.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uiForm) ;
      jcrBrowser.setSession(session);      
      uiForm.setRenderedSibling(JcrBrowser.class);      
    }
  }  
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIReferenceForm uiForm = (UIReferenceForm) event.getComponent() ;
      uiForm.setRenderedSibling(UIContentConfig.class) ;
    }
  }  

}
