package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.nodetype.NodeType;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIDateInput;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;

public class UIPropertyForm extends UISimpleForm implements ChangeNodeListener {

  public static final String VALUE = "value";

  public static final String CHANGE_TYPE_ACTION = "changeType";

  protected Node node_;
  protected UISelectBox typeInput_;

  public UIPropertyForm() throws Exception {
    super("propertyForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    List<SelectItem> options = new ArrayList<SelectItem>();
    options.add(new SelectItem(PropertyType.TYPENAME_BINARY, Integer
        .toString(PropertyType.BINARY)));
    options.add(new SelectItem(PropertyType.TYPENAME_BOOLEAN, Integer
        .toString(PropertyType.BOOLEAN)));
    options.add(new SelectItem(PropertyType.TYPENAME_DATE, Integer
        .toString(PropertyType.DATE)));
    options.add(new SelectItem(PropertyType.TYPENAME_DOUBLE, Integer
        .toString(PropertyType.DOUBLE)));
    options.add(new SelectItem(PropertyType.TYPENAME_LONG, Integer
        .toString(PropertyType.LONG)));
    options.add(new SelectItem(PropertyType.TYPENAME_NAME, Integer
        .toString(PropertyType.NAME)));
    options.add(new SelectItem(PropertyType.TYPENAME_PATH, Integer
        .toString(PropertyType.PATH)));
    options.add(new SelectItem(PropertyType.TYPENAME_REFERENCE, Integer
        .toString(PropertyType.REFERENCE)));
    options.add(new SelectItem(PropertyType.TYPENAME_STRING, Integer
        .toString(PropertyType.STRING)));
    options.add(new SelectItem(PropertyType.TYPENAME_UNDEFINED, Integer
        .toString(PropertyType.UNDEFINED)));

    UIStringInput nameInput_ = new UIStringInput("name", "");
    add(nameInput_);
    typeInput_ = new UISelectBox("type", "", options);
    typeInput_.setUpdateOnChangeAction(CHANGE_TYPE_ACTION);
    typeInput_.setValue(Integer.toString(PropertyType.STRING));
    add(typeInput_);
    UIInput valueInput_ = new UIStringInput(VALUE, "");
    add(valueInput_);

    addActionListener(ChangeTypeActionListener.class, CHANGE_TYPE_ACTION);
    addActionListener(SavePropertyActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    setRendered(false);
  }

  static public class ChangeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertyForm uiForm = (UIPropertyForm) event.getSource();
      int type = Integer.parseInt(uiForm.getUIInput("type").getValue());
      uiForm.remove(VALUE);
      if (PropertyType.BINARY == type) {
        uiForm.add(new UIUploadInput(VALUE));
      } else if (PropertyType.BOOLEAN == type) {
        uiForm.add(new UICheckBox(VALUE, "quoted"));
      } else if (PropertyType.DATE == type) {
        uiForm.add(new UIDateInput(VALUE, new Date()));
      } else {
        uiForm.add(new UIStringInput(VALUE, ""));
      }
    }
  }

  static public class SavePropertyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertyForm uiForm = (UIPropertyForm) event.getSource();
      String name = uiForm.getUIInput("name").getValue();
      Information iprovider = findInformationProvider(uiForm);
      try {
        NodeType nodetype = uiForm.node_.getPrimaryNodeType();
        if (uiForm.isMultiValue(VALUE)) {
          Value[] values = createValues(uiForm);
          if (nodetype.canSetProperty(name, values)) {
            uiForm.node_.setProperty(name, values);
            uiForm.setRenderedSibling(UIPropertiesInfo.class);
            uiForm.reset();
            uiForm.typeInput_.setValue(Integer.toString(PropertyType.STRING));
            return;
          }
        } else {
          Value value = createValue(uiForm);
          if (nodetype.canSetProperty(name, value)) {
            uiForm.node_.setProperty(name, value);
            uiForm.setRenderedSibling(UIPropertiesInfo.class);
            uiForm.reset();
            uiForm.typeInput_.setValue(Integer.toString(PropertyType.STRING));
            return;
          }
        }
        iprovider.addMessage(new ExoFacesMessage(
            "#{UIPropertyForm.msg.unsetable}"));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }

    public Value createValue(UIPropertyForm uiForm) throws Exception {
      UIInput uiInput = uiForm.getUIInput(VALUE);
      int type = Integer.parseInt(uiForm.getUIInput("type").getValue());
      ValueFactory valueFactory = uiForm.node_.getSession().getValueFactory();
      return createValue(valueFactory, uiInput, type);
    }

    public Value createValue(ValueFactory valueFactory, UIInput uiInput,
        int type) throws Exception {
      if (uiInput instanceof UIDateInput)
        return valueFactory.createValue(((UIDateInput) uiInput).getCalendar());
      else if (uiInput instanceof UICheckBox)
        return valueFactory.createValue(((UICheckBox) uiInput).getSelect());
      else {
        String valueString = uiInput.getValue();
        return valueFactory.createValue(valueString, type);
      }
    }

    public Value[] createValues(UIPropertyForm uiForm) throws Exception {
      ValueFactory valueFactory = uiForm.node_.getSession().getValueFactory();
      int type = Integer.parseInt(uiForm.getUIInput("type").getValue());
      List<UIInput> uiInputs = uiForm.getMultipleValuesUIInput(VALUE);
      Value[] values = new Value[uiInputs.size()];
      int i = 0;
      for (Iterator iter = uiInputs.iterator(); iter.hasNext(); i++) {
        UIInput uiInput = (UIInput) iter.next();
        values[i] = createValue(valueFactory, uiInput, type);
      }
      return values;
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertyForm uiForm = (UIPropertyForm) event.getSource();
      uiForm.reset();
      uiForm.typeInput_.setValue(Integer.toString(PropertyType.STRING));
      uiForm.setRenderedSibling(UIPropertiesInfo.class);
    }
  }

}
