package org.exoplatform.portlets.content.rules.component;

import javax.jcr.NodeIterator;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.component.UIWorkspaceBrowser;
import org.exoplatform.services.cms.rules.RuleService;
import org.exoplatform.services.cms.scripts.ScriptService;

public class UIRuleList extends UIExoCommand{
  
  final static public String ADD_ACTION = "add".intern();  
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;

  private RuleService ruleService_;
  private boolean hasRules;
  
  public UIRuleList(RuleService ruleService) {
    setId("UIRuleList") ;
    setRendererType("VelocityRenderer") ;
    
    ruleService_ = ruleService;
    
    UIToolbar toolbar = new UIToolbar("RuleToolbar"); 
    getChildren().add(toolbar);
    toolbar.addLeftButton(new Button("#{UIRuleList.button.add}", ADD_PARAMS));    
    toolbar.addRightButton(new Button("#{UIRuleList.button.cancel}", CANCEL_PARAMS));
    
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
  }

  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  
  public boolean hasRules() throws Exception{
     return ruleService_.hasRules();
  } 
  
  public NodeIterator getRules() throws Exception{
    return ruleService_.getRules();
  }

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      uicomp.setRenderedSibling(UIRuleForm.class);
      UIRuleForm ruleForm = (UIRuleForm) uicomp.getSibling(UIRuleForm.class);
      ruleForm.setRuleName(null);
    }
  }    
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      uicomp.setRenderedSibling(UIECMAdminSummary.class);
    }
  }    

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      UIRuleForm ruleForm = (UIRuleForm) uicomp.getSibling(UIRuleForm.class);
      ruleForm.setRuleName(name);
      uicomp.setRenderedSibling(UIRuleForm.class);
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      uicomp.ruleService_.removeRule(name);   
    }
  }        
}
