package org.exoplatform.portlets.content.script.component;

import javax.jcr.NodeIterator;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.scripts.ScriptService;

public class UIScriptList extends UIExoCommand{
  
  final static public String ADD_ACTION = "add".intern();  
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;

  protected ScriptService scriptService_;
  
  public UIScriptList(ScriptService scriptService) {
    setId("UIScriptList") ;
    setRendererType("VelocityRenderer") ;
    
    scriptService_ = scriptService;
    
    UIToolbar toolbar = new UIToolbar("ScriptToolbar"); 
    getChildren().add(toolbar);
    toolbar.addLeftButton(new Button("#{UIScriptList.button.add}", ADD_PARAMS));    
    toolbar.addRightButton(new Button("#{UIScriptList.button.cancel}", CANCEL_PARAMS));
    
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
  }

  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  
  public boolean hasScripts() throws Exception{
     return scriptService_.hasScripts();
  } 
  
  public NodeIterator getScripts() throws Exception{
    return scriptService_.getScripts();
  }

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptList uicomp = (UIScriptList) event.getComponent() ;
      uicomp.setRenderedSibling(UIScriptForm.class);
      UIScriptForm scriptForm = (UIScriptForm) uicomp.getSibling(UIScriptForm.class);
      scriptForm.setScriptName(null);
    }
  }    
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptList uicomp = (UIScriptList) event.getComponent() ;
      uicomp.setRenderedSibling(UIECMAdminSummary.class);
    }
  }    

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptList uicomp = (UIScriptList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      UIScriptForm scriptForm = (UIScriptForm) uicomp.getSibling(UIScriptForm.class);
      scriptForm.setScriptName(name);
      uicomp.setRenderedSibling(UIScriptForm.class);
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptList uicomp = (UIScriptList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      uicomp.scriptService_.removeScript(name);   
    }
  }        
}
