/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UITemplatesList extends UIExoCommand {
  
  static public String DELETE_TEMPLATE = "deleteTemplate" ;
  static public String EDIT_TEMPLATE = "editTemplate" ;
  
  static Parameter[] DELETE_TEMPLATE_PARAMS = { new Parameter(ACTION , DELETE_TEMPLATE)} ;
  static Parameter[] EDIT_TEMPLATE_PARAMS = { new Parameter(ACTION , EDIT_TEMPLATE)} ;  
  
  protected String nodetype;
  protected boolean isDialog_;
  protected TemplateService templateService_;
  
  public UITemplatesList(TemplateService templateService) throws Exception {
    setId("UITemplatesList") ;
    setRendererType("VelocityRenderer") ;
    
    templateService_ = templateService;
    
    addActionListener(DeleteTemplateActionListener.class, DELETE_TEMPLATE)  ;
    addActionListener(EditTemplateActionListener.class, EDIT_TEMPLATE)  ;
  }
  
  public void setNodeType(String nodeType) {
    this.nodetype = nodeType;
  }
  
  public void setDialog(boolean isDialog) {
    isDialog_ = isDialog;
  }
  
  public NodeIterator getTemplateIterator() throws Exception {
    return templateService_.getTemplates(isDialog_, nodetype);
  }
  
  public Parameter[] getDeleteTemplateParams() { return DELETE_TEMPLATE_PARAMS ; }
  public Parameter[] getEditTemplateParams() { return EDIT_TEMPLATE_PARAMS ; }
  
  static public class DeleteTemplateActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplatesList uicomp = (UITemplatesList) event.getComponent() ;
      String templateName = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);      
      for (int i = 0; i < TemplateService.UNDELETABLE_TEMPLATES.length; i++) {
        String template = TemplateService.UNDELETABLE_TEMPLATES[i];
        if(template.equals(templateName)){
          iprovider.addMessage(new ExoFacesMessage("#{UITemplatesList.msg.undeletable}")) ;          
          return;
        }
      }
      try {
        uicomp.templateService_.removeTemplate(uicomp.isDialog_, uicomp.nodetype, templateName);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class EditTemplateActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplatesList uicomp = (UITemplatesList) event.getComponent() ;
      String templateName = event.getParameter(OBJECTID);
      UITemplatesManager manager = (UITemplatesManager) uicomp.getAncestorOfType(UITemplatesManager.class);
      UITemplateForm templateForm = (UITemplateForm) manager.getChildComponentOfType(UITemplateForm.class);
      templateForm.editTemplate(uicomp.isDialog_, uicomp.nodetype, templateName);
      manager.setRenderedComponent(UITemplateForm.class);      
    }
  }
  
}
