/*
 * Created on Mar 18, 2005
 */
package org.exoplatform.services.cms.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.picocontainer.Startable;

/**
 * @author benjaminmestrallet
 */
public class CmsConfigurationServiceImpl implements CmsConfigurationService, Startable {

  private RepositoryService jcrService_;

  private CmsConfig config_;

  public CmsConfigurationServiceImpl(InitParams params,
      RepositoryService jcrService) throws Exception {
    jcrService_ = jcrService;
    config_ = (CmsConfig) params.getObjectParamValues(CmsConfig.class).get(0);    
  }
  
  public void start() {
    try {
      init();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void stop() {
  }  

  private void init() throws Exception {
    ManageableRepository jcrRepository = jcrService_.getRepository();
    Map<String, String[]> permissions = null;
    List jcrPaths = config_.getJcrPaths();
    for (Iterator iter = jcrPaths.iterator(); iter.hasNext();) {
      CmsConfig.JcrPath jcrPath = (CmsConfig.JcrPath) iter.next();
      List workspaces = jcrPath.getWorkspaces();
      for (Iterator iterator = workspaces.iterator(); iterator.hasNext();) {
        CmsConfig.Workspace workspace = (CmsConfig.Workspace) iterator.next();
        Session session = jcrRepository.getSystemSession(workspace.getName());        
        permissions =  new HashMap<String, String[]>();
        List perms = workspace.getPermissions();
        for (Iterator iterator2 = perms.iterator(); iterator2.hasNext();) {
          CmsConfig.Permission perm = (CmsConfig.Permission) iterator2.next();          
          List<String> permsList = new ArrayList<String>();
          if("true".equals(perm.getRead()))
            permsList.add(PermissionType.READ);
          if("true".equals(perm.getAddNode()))
            permsList.add(PermissionType.ADD_NODE);
          if("true".equals(perm.getSetProperty()))
            permsList.add(PermissionType.SET_PROPERTY);
          if("true".equals(perm.getRemove()))
            permsList.add(PermissionType.REMOVE);          
          String[] permsArray = new String[permsList.size()];
          permissions.put(perm.getOwner(), permsList.toArray(permsArray));
        }
        Utils.makePath(session.getRootNode(), jcrPath.getPath(), "nt:unstructured", permissions);
        session.save();
      }           
    }
  }

  public String getWorkspace() {
    return config_.getWorkspace();
  }

  public String getDraftWorkspace() {
    return config_.getDraftWorkspace();
  }
  
  public String getBackupWorkspace() {
    return config_.getBackupWorkspace();
  }

  public String getContentLocation() {
    return config_.getContentLocation();
  }

  public String getJcrPath(String alias) {
    List jcrPaths = config_.getJcrPaths();
    for (Iterator iter = jcrPaths.iterator(); iter.hasNext();) {
      CmsConfig.JcrPath jcrPath = (CmsConfig.JcrPath) iter.next();
      if (jcrPath.getAlias().equals(alias)) {
        return jcrPath.getPath();
      }
    }
    return null;
  }

  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof AddPathPlugin) {
      AddPathPlugin impl = (AddPathPlugin) plugin;

      try {
        ManageableRepository jcrRepository = jcrService_.getRepository();
        Session session = jcrRepository.getSystemSession(config_.getWorkspace());
        Node root = session.getRootNode();
        List paths = impl.getPaths();
        for (Iterator iter = paths.iterator(); iter.hasNext();) {
          String path = (String) iter.next();
          Utils.makePath(root, path, "nt:unstructured", Utils.getReadPermissions());
        }
        root.save();
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return null;
  }

}