/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.component ;

import java.util.ArrayList ;
import java.util.Calendar ;
import java.util.GregorianCalendar ;
import java.util.List ;
import java.util.Map ;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId ;
import javax.jcr.Node ;

import org.exoplatform.commons.utils.ISO8601 ;
import org.exoplatform.faces.FacesConstants ;
import org.exoplatform.faces.cms.CMSUtils ;
import org.exoplatform.faces.context.PortletExternalContext;
import org.exoplatform.faces.core.Util ;
import org.exoplatform.faces.core.component.UIDateTimeInput ;
import org.exoplatform.faces.core.component.UISimpleForm ;
import org.exoplatform.faces.core.event.ExoActionEvent ;
import org.exoplatform.faces.core.event.ExoActionListener ;
import org.exoplatform.services.calendar.CalendarService ;
import org.exoplatform.services.cms.JcrInputProperty ;
import org.exoplatform.services.cms.templates.TemplateService ;


public class UICalendarEventForm extends UISimpleForm {

	private CalendarService calendarService_ ;
	private TemplateService templateService_ ;
	private boolean displayCheckInputMsg_ ;
	private List errorMsgList_ ;
	private boolean editMode_  ;
	private String eventId_ ;
	private Node eventNode_ ;

	public UICalendarEventForm(TemplateService templateService, CalendarService service)  
						throws Exception {

		super("calendarEventForm", "post", null) ;
		setRendererType("SimpleFormVelocityRenderer")  ;

		this.templateService_ = templateService ;
		setId("UICalendarEventForm") ;
		this.calendarService_ = service ;
		this.displayCheckInputMsg_ = false ;
		this.editMode_ = false ;
		this.eventId_ = null ;
		this.eventNode_ = null ;

		addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
		addActionListener(SaveActionListener.class, SAVE_ACTION) ;
		addActionListener(DeleteActionListener.class, DELETE_ACTION) ;

		addActionListener(EditActionListener.class, EDIT_ACTION) ;

		String action = new StringBuffer("change_dtstart").append(UIDateTimeInput.DAY_EXTENSION).toString() ;
		addActionListener(ChangeDtStartActionListener.class, action) ;
		action = new StringBuffer("change_dtstart").append(UIDateTimeInput.MONTH_EXTENSION).toString() ;
		addActionListener(ChangeDtStartActionListener.class, action) ;
		action = new StringBuffer("change_dtstart").append(UIDateTimeInput.YEAR_EXTENSION).toString() ;
		addActionListener(ChangeDtStartActionListener.class, action) ;
		action = new StringBuffer("change_dtstart").append(UIDateTimeInput.HOUR_EXTENSION).toString() ;
		addActionListener(ChangeDtStartActionListener.class, action) ;
		action = new StringBuffer("change_dtstart").append(UIDateTimeInput.MINUTE_EXTENSION).toString() ;
		addActionListener(ChangeDtStartActionListener.class, action) ;
	}

	public CalendarService getCalendarService() {
		return this.calendarService_ ;
	}
	
	public String getEventId() {
		return this.eventId_ ;
	}

	public void setEventId(String eventId) {
		UICalendar uiCalendar = (UICalendar) this.getAncestorOfType(UICalendar.class) ;
		if (eventId == null || "".equals(eventId)) {
			this.eventNode_ = null ;
		}
		else {
			try {
				this.eventNode_ = this.calendarService_.getCalendarEventByPath(eventId) ;
			} catch (Exception e) {
				this.eventId_ = null ;
				this.eventNode_ = null ;
			}
		}
		this.eventId_ = eventId ;
	}

	public String getDialogPath() {    
		try {
			return this.templateService_.getDefaultTemplatePath(true, "exo:calendarEvent") ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}    

	public String getViewPath() {    
		try {
			return this.templateService_.getDefaultTemplatePath(false, "exo:calendarEvent") ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	public Node getNode() {
		return this.eventNode_ ;
	}
	
	public String getEditAction() { return EDIT_ACTION  ; }

	public String getDeleteAction() { return DELETE_ACTION  ; }

	public void clean() {
		clearInputs() ;
		getChildren().clear() ;
		this.displayCheckInputMsg_ = false ;
		this.eventId_ = "" ;
	}
	
	public String formatDate(Calendar cal) {
		GregorianCalendar gCal = (GregorianCalendar) cal ;
		return ISO8601.format(gCal) ;
	}

	public List getErrorMsgList() {
		return this.errorMsgList_ ;
	}

	public void setErrorMsgList(List errorMsgList) {
		this.errorMsgList_ = errorMsgList ;
	}

	public boolean isEditMode() {
		return this.editMode_ ;
	}

	public void setEditMode(boolean editMode) {
		this.editMode_ = editMode ;
	}
	
	public boolean isCheckInputMsgDisplayed() {
		return this.displayCheckInputMsg_ ;
	}

	public void setDisplayCheckInputMsg(boolean displayCheckInputMsg) {
		this.displayCheckInputMsg_ = displayCheckInputMsg ;
	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarEventForm uiForm = (UICalendarEventForm) event.getComponent() ;
			UICalendarPortlet calendarPortlet =
				(UICalendarPortlet) uiForm.getAncestorOfType(UICalendarPortlet.class) ;
			calendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) calendarPortlet.getChildComponentOfType(
					UIToolbarPanel.class)).setRendered(true) ;
		}
	}

	static public class EditActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {

			UICalendarEventForm uiCalendarEventForm = (UICalendarEventForm) event.getComponent() ;
			uiCalendarEventForm.setDisplayCheckInputMsg(false) ;
			uiCalendarEventForm.setEditMode(true) ;
			UICalendarPortlet calendarPortlet = (UICalendarPortlet) uiCalendarEventForm.getParent() ;
			calendarPortlet.setRenderedComponent(UICalendarEventForm.class) ;
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
	    PortletExternalContext econtext =  
	    	(PortletExternalContext) FacesContext.getCurrentInstance().getExternalContext();
	    ResourceBundle res = econtext.getApplicationResourceBundle() ;
			FacesContext context = FacesContext.getCurrentInstance() ; 
			String user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
			if (user == null) user = "anonymous" ;

			UICalendarEventForm uiCalendarEventForm = (UICalendarEventForm) event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendarEventForm.getParent() ;
			Map rawInputs = CMSUtils.prepareMap(uiCalendarEventForm.getUIInputs()) ;
			JcrInputProperty property ;
			List errorMsgList = new ArrayList(5) ;
			boolean inputOk = true ;
			// Check input
			// 1) Summary not null
			property = (JcrInputProperty) rawInputs.get("/node/exo:summary") ;
			if (property == null || "".equals(property.getValue())) {
				inputOk = false ;
				errorMsgList.add(res.getString("UICalendarEventForm.msg.subject-error")) ;
			}
			// 2) start / end date coherence
			property = (JcrInputProperty) rawInputs.get("/node/exo:dtstart") ;
			if (property == null || "".equals(property.getValue())) {
				inputOk = false ;
				errorMsgList.add(res.getString("UICalendarEventForm.msg.date-error")) ;
			}
			else {
				GregorianCalendar dtstart = (GregorianCalendar) ISO8601.parse((String)property.getValue()) ;
				property = (JcrInputProperty) rawInputs.get("/node/exo:dtend") ;
				if (property == null || "".equals(property.getValue())) {
					inputOk = false ;
					errorMsgList.add(res.getString("UICalendarEventForm.msg.date-error")) ;
				}
				else {
					GregorianCalendar dtend = (GregorianCalendar) ISO8601.parse((String)property.getValue()) ;
					if (dtstart == null | dtend == null) {
						inputOk = false ;
						errorMsgList.add(res.getString("UICalendarEventForm.msg.date-error")) ;
					}
					else {
						if (dtend.before(dtstart)) {
							inputOk = false ;
							errorMsgList.add(res.getString("UICalendarEventForm.msg.date-incoherence")) ;
						}
					}
				}
			}
			if (inputOk) {
				Node eventNode = uiCalendarEventForm.getNode() ;
				if (eventNode != null) {
					String oldShareType = eventNode.getProperty("exo:sharetype").getValue().getString() ;
			    property  =  (JcrInputProperty) rawInputs.get("/node/exo:sharetype") ;
			    String newShareType = (String) property.getValue() ;
			    if (newShareType != oldShareType) {
			    	uiCalendarEventForm.calendarService_.removeCalendarEvent(user, eventNode.getName()) ;
			    }
				}
				uiCalendarEventForm.calendarService_.saveCalendarEvent(user, uiCalendarPortlet.getCurrentCalendarType(),
						uiCalendarPortlet.getCurrentCalendar(), rawInputs) ;

				uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
				((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
			}
			else {
				uiCalendarEventForm.setDisplayCheckInputMsg(true) ;
				uiCalendarEventForm.setErrorMsgList(errorMsgList) ;
				uiCalendarPortlet.setRenderedComponent(UICalendarEventForm.class) ;
			}
		}
	}

	static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {

			String user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
			if (user == null) user = "anonymous" ;

			UICalendarEventForm uiCalendarEventForm = (UICalendarEventForm) event.getComponent() ;
			UICalendarPortlet calendarPortlet = (UICalendarPortlet) uiCalendarEventForm.getParent() ;
			Node eventNode = uiCalendarEventForm.getNode() ;
			if (eventNode == null) {
					String eventId = uiCalendarEventForm.getEventId() ;
		    	uiCalendarEventForm.calendarService_.removeCalendarEvent(user, eventId) ;
			}
			else {
	    	uiCalendarEventForm.calendarService_.removeCalendarEvent(eventNode) ;
				
			}
			uiCalendarEventForm.setEventId(null) ;
			calendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) calendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}
	
	static public class ChangeDtStartActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarEventForm uiCalendarEventForm = (UICalendarEventForm) event.getComponent() ;
			UIDateTimeInput uiDtStart = (UIDateTimeInput)uiCalendarEventForm.getUIInput("dtstart") ;
			UIDateTimeInput uiDtEnd = (UIDateTimeInput)uiCalendarEventForm.getUIInput("dtend") ;
			if (uiDtStart != null && uiDtEnd != null) {
				if (uiDtEnd.getCalendar().before(uiDtStart.getCalendar())) {
					GregorianCalendar gCal = (GregorianCalendar)uiDtStart.getCalendar().clone() ;
					uiDtEnd.addTime(uiDtStart.getCalendar().getTime()) ;
					
					return ;
				}
			}
		}
	}
}
