/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.component;

import java.util.GregorianCalendar;

import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;


public class UIToolbarPanel extends UIExoCommand {

	private static Parameter [] NEW_EVENT_PARAM = {new Parameter(ACTION, UICalendarPortlet.NEW_EVENT_ACTION)};

	private static Parameter [] MOVE_HOME_PARAM = {new Parameter(ACTION, UICalendarPortlet.MOVE_HOME_ACTION)};
	private static Parameter [] VIEW_DAY_PARAM = {new Parameter(ACTION, UICalendarPortlet.VIEW_DAY_ACTION)};
	private static Parameter [] VIEW_WEEK_PARAM = {new Parameter(ACTION, UICalendarPortlet.VIEW_WEEK_ACTION)};
	private static Parameter [] VIEW_MONTH_PARAM = {new Parameter(ACTION, UICalendarPortlet.VIEW_MONTH_ACTION)};

	private static Parameter [] TEST_SERVICE_PARAM = {new Parameter(ACTION, "TEST_SERVICE")};
	
	public UIToolbarPanel() {
		super();
	  setId("UIToolbarPanel");
	  setRendererType("VelocityRenderer") ;
		
		addActionListener(MoveHomeActionListener.class, UICalendarPortlet.MOVE_HOME_ACTION);
		addActionListener(NewEventActionListener.class, UICalendarPortlet.NEW_EVENT_ACTION);
		addActionListener(ViewDayActionListener.class, UICalendarPortlet.VIEW_DAY_ACTION);
		addActionListener(ViewWeekActionListener.class, UICalendarPortlet.VIEW_WEEK_ACTION);
		addActionListener(ViewMonthActionListener.class, UICalendarPortlet.VIEW_MONTH_ACTION);
		
		addActionListener(testServiceActionListener.class, "TEST_SERVICE");
	}
 
	public String getViewMode() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class);
		return uiCalendarPortlet.getViewMode();
	}

	public boolean isDayViewMode() {
		if (getViewMode().equals(UICalendarPortlet.DAY_VIEW_MODE)) return true;
		else return false;
	}

	public boolean isWeekViewMode() {
		if (getViewMode().equals(UICalendarPortlet.WEEK_VIEW_MODE)) return true;
		else return false;
	}
	
	public boolean isMonthViewMode() {
		if (getViewMode().equals(UICalendarPortlet.MONTH_VIEW_MODE)) return true;
		else return false;
	}

	public boolean isUserLoggedIn() {
		String remoteUser = 
	      FacesContext.getCurrentInstance().getExternalContext().getRemoteUser();
	      return !(remoteUser == null || "".equals(remoteUser));
	}

	public boolean showTestService() {
		String remoteUser = 
	      FacesContext.getCurrentInstance().getExternalContext().getRemoteUser();
	    if (remoteUser != null && "fichatel".equals(remoteUser)) return true;
	    return false;
	}
	
	public Parameter[] getNewEventParams() { return NEW_EVENT_PARAM; }
	public Parameter[] getMoveHomeParams() { return MOVE_HOME_PARAM; }
	public Parameter[] getViewDayParams() { return VIEW_DAY_PARAM; }
	public Parameter[] getViewWeekParams() { return VIEW_WEEK_PARAM; }
	public Parameter[] getViewMonthParams() { return VIEW_MONTH_PARAM; }
	
	public Parameter[] getTestServiceParams() { return TEST_SERVICE_PARAM; }
	
	static public class NewEventActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			UICalendarEventForm uiCalendarEventForm = (UICalendarEventForm) uiCalendarPortlet.getChildComponentOfType(UICalendarEventForm.class);
			uiCalendarEventForm.clean();
			uiCalendarEventForm.setEditMode(true);
			uiCalendarPortlet.setRenderedComponent(UICalendarEventForm.class);
		}
	}

	static public class MoveHomeActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			uiCalendarPortlet.setSelectDate(new GregorianCalendar());
			uiCalendarPortlet.setViewMode(UICalendarPortlet.MONTH_VIEW_MODE);
			uiCalendarPortlet.setRenderedComponent(UICalendar.class);
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true);
		}
	}

	static public class ViewDayActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			String viewType = uiCalendarPortlet.getViewMode();
			if (viewType.equals(UICalendarPortlet.DAY_VIEW_MODE)) {
				return;
			}
			uiCalendarPortlet.setViewMode(UICalendarPortlet.DAY_VIEW_MODE);
			uiCalendarPortlet.setRenderedComponent(UICalendar.class);
			panel.setRendered(true);
		}
	}

	static public class ViewWeekActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			String viewType = uiCalendarPortlet.getViewMode();
			if (viewType.equals(UICalendarPortlet.WEEK_VIEW_MODE)) {
				return;
			}
			uiCalendarPortlet.setViewMode(UICalendarPortlet.WEEK_VIEW_MODE);
			uiCalendarPortlet.setRenderedComponent(UICalendar.class);
			panel.setRendered(true);
		}
	}

	static public class ViewMonthActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			String viewType = uiCalendarPortlet.getViewMode();
			if (viewType.equals(UICalendarPortlet.MONTH_VIEW_MODE)) {
				return;
			}
			uiCalendarPortlet.setViewMode(UICalendarPortlet.MONTH_VIEW_MODE);
			uiCalendarPortlet.setRenderedComponent(UICalendar.class);
			panel.setRendered(true);
		}
	}

	static public class testServiceActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIToolbarPanel panel = (UIToolbarPanel) event.getComponent();
			UICalendarPortlet calendarPortlet = (UICalendarPortlet) panel.getAncestorOfType(UICalendarPortlet.class);
			UICalendar uiCalendar = (UICalendar) calendarPortlet.getChildComponentOfType(UICalendar.class);
			uiCalendar.findCalendarEventByUserAndDate(new GregorianCalendar());
		}
	}
	
}
