package org.exoplatform.portlets.calendar.renderer.html;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.calendar.component.UICalendarPortlet;
import org.exoplatform.portlets.calendar.component.UIMiniCalendar;

public class MiniCalendarRenderer extends HtmlBasicRenderer {

	// Effective ui parameters
	private Parameter dayParam_ = new Parameter(UICalendarPortlet.DAY_PARAM, "");
	private Parameter monthParam_ = new Parameter(UICalendarPortlet.MONTH_PARAM, "");
	private Parameter yearParam_ = new Parameter(UICalendarPortlet.YEAR_PARAM, "");
	private Parameter periodMovedParam_ = new Parameter(UICalendarPortlet.PERIOD_MOVED, "");

	// Rendered ui parameters
	private Parameter[] viewDayParams_ = { UICalendarPortlet.VIEW_DAY_PARAM, dayParam_, monthParam_, yearParam_ };
	private Parameter[] viewMonthParams_ = { UICalendarPortlet.VIEW_MONTH_PARAM, dayParam_, monthParam_, yearParam_ };

	private Parameter[] movePreviousParams_ = { UICalendarPortlet.MOVE_PREVIOUS_PARAM, periodMovedParam_ };
	private Parameter[] moveNextParams_ = { UICalendarPortlet.MOVE_NEXT_PARAM, periodMovedParam_ };

	final public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		ResourceBundle res = getApplicationResourceBundle(context.getExternalContext());
		ResponseWriter w = context.getResponseWriter();
		UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) component;
		
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
		
		SimpleDateFormat dateFormat = new SimpleDateFormat();
		w.write("<div class='" + uiMiniCalendar.getClazz() +"'") ;
		w.write("<table>");
		w.write("<tr>");
		w.write("<td class='period_title'>");
		if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.DAY_VIEW_MODE) ||
				uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.WEEK_VIEW_MODE)) {
			periodMovedParam_.setValue(UICalendarPortlet.MOVE_MONTH);
			buttonRenderer_.render(w, uiMiniCalendar, res.getString("UICalendarPortlet.button.previous"), movePreviousParams_);
			dateFormat.applyPattern("MMMM, yyyy");
			w.write(dateFormat.format(uiCalendarPortlet.getSelectDate().getTime()));
			buttonRenderer_.render(w, uiMiniCalendar, res.getString("UICalendarPortlet.button.next"), moveNextParams_);
		} else if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.MONTH_VIEW_MODE)) {
			periodMovedParam_.setValue(UICalendarPortlet.MOVE_YEAR);
			buttonRenderer_.render(w, uiMiniCalendar, res.getString("UICalendarPortlet.button.previous"), movePreviousParams_);
			dateFormat.applyPattern("yyyy");
			w.write(dateFormat.format(uiCalendarPortlet.getSelectDate().getTime()));
			buttonRenderer_.render(w, uiMiniCalendar, res.getString("UICalendarPortlet.button.next"), moveNextParams_);
		}
		w.write("</td>");
		w.write("</tr>");

		// Mini calandar
		w.write("<tr>");
		w.write("<td>");
		if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.DAY_VIEW_MODE) ||
				uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.WEEK_VIEW_MODE)) {
			addMiniCalendarByMonth(w, res, uiMiniCalendar);
			
		} else if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.MONTH_VIEW_MODE)) {
			addMiniCalendarByYear(w, res, uiMiniCalendar);
		}
		w.write("</td>");
		w.write("</tr>");

		// Today's date
//		w.write("</tr>");
		w.write("<tr>");
		w.write("<td>");
		dateFormat.applyPattern("EEEEEEEE, MMM dd");
		w.write(res.getString("UICalendarPortlet.label.today") + " : "); 
		w.write("<span class='blueTxt'>" + dateFormat.format(GregorianCalendar.getInstance().getTime()) + "</span>");
		w.write("</td>");
		w.write("</tr>");
		w.write("</table>");
		w.write("</div>") ;
	}

	private void addMiniCalendarByMonth(ResponseWriter w, ResourceBundle res, UIMiniCalendar uiMiniCalendar) throws IOException {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
		SimpleDateFormat dateFormat = new SimpleDateFormat("EEE");
		GregorianCalendar gCal = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		gCal.add(Calendar.DATE, Calendar.MONDAY - gCal.get(Calendar.DAY_OF_WEEK));

		w.write("<table class='MiniCalendarByMonth'>");
		w.write("<tr>");
		String strDay;
		for (int i = 0; i < 7; i++) {
			w.write("<th>");
			try {
				strDay = dateFormat.format(gCal.getTime()).substring(0, 3);
			} catch (IndexOutOfBoundsException e) {
				strDay = dateFormat.format(gCal.getTime());
			}
			w.write(strDay);
			gCal.add(Calendar.DATE, 1);
			w.write("</th>");
		}
		w.write("</tr>");

		w.write("<tr class='underTH'>");
		w.write("<td colspan=7></td>");
		w.write("</tr>");
		
		GregorianCalendar startDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		startDate.set(Calendar.DATE, startDate.getActualMinimum(Calendar.DATE));
		if (startDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) startDate.add(Calendar.DATE, -6); 
		else startDate.add(Calendar.DATE, Calendar.MONDAY - startDate.get(Calendar.DAY_OF_WEEK));

		GregorianCalendar endDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		endDate.set(Calendar.DATE, endDate.getActualMaximum(Calendar.DATE));
		if (endDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) endDate.add(Calendar.DATE, 1); 
		else endDate.add(Calendar.DATE, Calendar.SATURDAY - endDate.get(Calendar.DAY_OF_WEEK) + Calendar.SUNDAY);
		
		GregorianCalendar currentDate = (GregorianCalendar) startDate.clone();
		GregorianCalendar today = new GregorianCalendar();
		GregorianCalendar selectDate = uiCalendarPortlet.getSelectDate();
		dateFormat.applyPattern("d");
		boolean lastDay = false;
		while (!lastDay) {
			w.write("<tr>");
			for (int i = 0; i < 7; i++) {
				if (currentDate.get(Calendar.MONTH) != selectDate.get(Calendar.MONTH)) {
					w.write("<td class='outOfMonth'>");
				} else if (currentDate.get(Calendar.YEAR) == today.get(Calendar.YEAR)
						&& currentDate.get(Calendar.MONTH) == today.get(Calendar.MONTH)
						&& currentDate.get(Calendar.DATE) == today.get(Calendar.DATE)) {
					
					w.write("<td class='today'>");
				} else if (currentDate.get(Calendar.YEAR) == selectDate.get(Calendar.YEAR)
						&& currentDate.get(Calendar.MONTH) == selectDate.get(Calendar.MONTH)
						&& currentDate.get(Calendar.DATE) == selectDate.get(Calendar.DATE)) {
					
					w.write("<td class='selected'>");
				}
				else w.write("<td>");
				dayParam_.setValue(new Integer(currentDate.get(Calendar.DATE)).toString());
				monthParam_.setValue(new Integer(currentDate.get(Calendar.MONTH)).toString());
				yearParam_.setValue(new Integer(currentDate.get(Calendar.YEAR)).toString());
				linkRenderer_.render(w, uiMiniCalendar, dateFormat.format(currentDate.getTime()), viewDayParams_);
				w.write("</td>");
				if (currentDate.get(Calendar.YEAR) == endDate.get(Calendar.YEAR)
						&& currentDate.get(Calendar.MONTH) == endDate.get(Calendar.MONTH)
						&& currentDate.get(Calendar.DATE) == endDate.get(Calendar.DATE)) {
					lastDay = true;
				}
				currentDate.add(Calendar.DATE, 1);
			}
			w.write("</tr>");
		}
		w.write("</table>");
	}
	
	private void addMiniCalendarByYear(ResponseWriter w, ResourceBundle res, UIMiniCalendar uiMiniCalendar) throws IOException {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
		SimpleDateFormat dateFormat = new SimpleDateFormat("MMM");
		int selectMonth = uiCalendarPortlet.getSelectMonth();
		int selectDay = uiCalendarPortlet.getSelectDate().get(Calendar.DATE);
		
		GregorianCalendar currentDate = new GregorianCalendar(uiCalendarPortlet.getSelectDate().get(Calendar.YEAR), 0, 1);
		currentDate.set(Calendar.DATE, uiCalendarPortlet.getSelectDate().get(Calendar.DATE));
		w.write("<table class='MiniCalendarByYear'>");
		for (int i =0; i < 3; i++) {
			w.write("<tr>");
			for (int j =0; j < 4; j++) {
				if (currentDate.get(Calendar.MONTH) == selectMonth) w.write("<td class='selected'>");
				else w.write("<td>");
				String strMonth;
				try {
					strMonth = dateFormat.format(currentDate.getTime()).substring(0, 3);
				} catch (IndexOutOfBoundsException e) {
					strMonth = dateFormat.format(currentDate.getTime());
				}
				if (selectDay > currentDate.getActualMaximum(Calendar.DATE))
					dayParam_.setValue(new Integer(selectDay).toString());
				else dayParam_.setValue(new Integer(currentDate.getActualMaximum(Calendar.DATE)).toString());
				monthParam_.setValue(new Integer(currentDate.get(Calendar.MONTH)).toString());
				yearParam_.setValue(new Integer(currentDate.get(Calendar.YEAR)).toString());
				linkRenderer_.render(w, uiMiniCalendar, strMonth , viewMonthParams_);
				currentDate.add(Calendar.MONTH, 1);
				w.write("</td>");
			}
			w.write("</tr>");
		}
		w.write("</table>");
	}
	
}
