/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import java.util.ResourceBundle;

import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPostForm.java,v 1.12 2004/10/20 23:39:59 benjmestrallet Exp $
 */
public class UIPostForm extends UISimpleForm {
  private UIStringInput subject_ ;
  private UITextArea    message_ ;
  Post post_ ;
  Topic topic_ ;
  Forum forum_ ; 
  
  public UIPostForm(ForumService service) throws Exception {
    super("postForm", "post", null) ;
    setId("UIPostForm") ;
    setClazz("UIPostForm");
    setRendererType("SimpleFormVelocityRenderer") ;
    message_ = new UITextArea("message", "") ;
    subject_ = new UIStringInput("subject", "") ;
    add(message_) ;
    add(subject_) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION)  ;
    addActionListener(SaveActionListener.class, SAVE_ACTION)  ;
  }
  
  public void setNewTopicInfo(Forum forum) {
    forum_ = forum ;
    topic_ = null ;
    post_ = null ;
    subject_.setText("") ;
    message_.setText("") ;
  }

  public void setNewPostInfo(Topic topic) {
    forum_ = null ;
    topic_ = topic ;
    post_ = null ;
    subject_.setText("") ;
    message_.setText("") ;
  }

  public void setQuotePostInfo(Topic topic, Post post) {
    forum_ = null ;
    topic_ = topic ;
    post_ = null ;
    subject_.setText(post.getSubject()) ;
    message_.setText(post.getMessage()) ;
  }

  public void setEditPostInfo(Topic topic, Post post) {
    forum_ = null ;
    topic_ = topic ;
    post_ = post ;
    subject_.setText(post.getSubject()) ;
    message_.setText(post.getMessage()) ;
  }
  
  public boolean canDecodeInvalidState()  { return forum_ != null  || topic_ != null; }
  
  static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPostForm uiForm = (UIPostForm) event.getComponent() ;
      UIForumPortlet forumPortlet = (UIForumPortlet)uiForm.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiForm);      
			if (uiForm.topic_ == null) 	{              
        forumPortlet.setRenderedComponent(UITopics.class);                        
      } else 	{
        forumPortlet.setRenderedComponent(UIPosts.class);        
      }
		}
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPostForm uiForm = (UIPostForm) event.getComponent() ;
      
      ForumService service = UIForumPortlet.getForumService() ;
      
			 String user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
       String subject = uiForm.subject_.getText() ;
       String message = uiForm.message_.getText() ;
       if(subject == null || subject.length() == 0) {
         ResourceBundle res = event.getApplicationResourceBundle() ;
         subject = res.getString("UIPostForm.msg.no-subject");
       }
       Topic topic = uiForm.topic_ ;
       boolean isNewTopic = false  ;
       if (topic == null) {
         topic = service.createTopicInstance() ;
         topic.setOwner(user) ;
         topic.setModifiedBy(user) ;
         topic.setTopic(subject) ;
         topic.setDescription("") ;
         service.addTopic(uiForm.forum_, topic) ;
         isNewTopic = true ;
       }
       
       boolean isNewPost = false ;
       Post post = uiForm.post_ ;
       if(post == null) {
         post = service.createPostInstance() ;
         post.setOwner(user) ;
         isNewPost = true ;
       }

       post.setModifiedBy(user) ;
       post.setSubject(subject) ;
       post.setMessage(message) ;
       post.setRemoteAddr("") ;
       if(isNewPost) service.addPost(topic, post) ;
       else  service.updatePost(post) ;
       UITopics uiTopics = (UITopics)uiForm.getSibling(UITopics.class) ;
       if(isNewTopic || isNewPost) {
         uiTopics.reload() ;
       }
			 uiTopics.visit( topic.getId() );
       UIPosts uiPosts = (UIPosts) uiForm.getSibling(UIPosts.class) ;
       uiPosts.setUIPostsData(uiTopics.getForum() , topic) ;
           
       UIForumPortlet forumPortlet = (UIForumPortlet)uiForm.getAncestorOfType(UIForumPortlet.class);
       forumPortlet.addHistoryElement(uiForm);      
       forumPortlet.setRenderedComponent(UIPosts.class);
		}
  }  
}
