/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.forum.ForumACL;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPosts.java,v 1.9 2004/10/20 18:46:31 benjmestrallet Exp $
 */
public class UIPosts extends UIExoCommand {
  private UIPageListIterator uiPageIterator_ ;
  private Topic topic_ ;
  private Forum forum_  ;
  private ForumACL acl_ ;  
  private WikiEngineService wikiEngineService_ ;
  private OrganizationService orgService_ ;
    
  private static final String REPLY = "reply";
  private static final String VIEW_FORUMS = "viewForums";          

  private static Parameter[] replyParams = { new Parameter(ACTION , REPLY)} ;
  private static Parameter[] viewForumParams = { new Parameter(ACTION , VIEW_FORUMS)} ;
  private static Parameter[] quoteParams = { new Parameter(ACTION , "quotePost")} ;
  private static Parameter[] editParams = { new Parameter(ACTION , "editPost")} ;
  private static Parameter[] viewTopicsParams = {new Parameter(ACTION , "viewTopics") } ; 
  public UIPosts(WikiEngineService weService, 
                 OrganizationService orgService,  
                 ForumService service, ForumACL acl) throws Exception {
    setRendererType("VelocityRenderer") ;
    setId("UIPosts") ;
    wikiEngineService_ = weService ;
    orgService_ = orgService ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_);
    acl_ = acl ;            
    
    addActionListener(ViewTopicsActionListener.class, "viewTopics")  ;
    addActionListener(ViewForumsActionListener.class, VIEW_FORUMS)  ;
    addActionListener(QuotePostActionListener.class, "quotePost")  ;
    addActionListener(EditPostActionListener.class, "editPost")  ;
    addActionListener(ReplyActionListener.class, REPLY)  ;
  }
  
  public Parameter[] getReplyParams(){ return replyParams;  }
  public Parameter[] getViewForumParams(){ return viewForumParams;  }
  public Parameter[] getQuoteParams(){ return quoteParams;  }
  public Parameter[] getEditParams(){ return editParams;  }
  public Parameter[] getViewTopicsParams(){ return viewTopicsParams;  }
  
  public String getForumName() { return forum_.getForumName(); }
   
  public Topic getTopic() { return topic_ ; } 

  public WikiEngineService getWikiEngineService() { return wikiEngineService_ ; }
  
  public OrganizationService  getOrganizationService() { return orgService_ ; }
  
  public void setUIPostsData(Forum forum, Topic topic) throws Exception {
    forum_ = forum ;
    topic_ = topic ; 
    ForumService service = UIForumPortlet.getForumService() ;
    uiPageIterator_.setPageList(service.getPosts(topic_.getId())) ;
  }
  
  public void reload() throws Exception {
    ForumService service = UIForumPortlet.getForumService() ;
    uiPageIterator_.setPageList(service.getPosts(forum_.getId())) ;
  }
  
  public List getPostLists(){
    return getUIPageIterator().getObjectInCurrentPage();
  }
  
  /**
   * Author: Long Nguyen Huu
   * 
   * @param signature
   * @return
   * @throws Exception
   */
  public String renderSignature(String signature) throws Exception {
    StringBuffer b = new StringBuffer() ;
    char[] buf = signature.toCharArray()  ;
    for(int i = 0; i < buf.length; i++) {
      switch(buf[i]) {
        case '\n' : b.append("<br/>") ; break ;
        case '\r' :  break ;
        case '&' : b.append("&amp;") ; break ;
        case '>' : b.append("&gt;") ; break ;
        case '<' : b.append("&lt;") ; break ;
        case ' ' : {
          b.append(" ") ;
          i++ ;
          while(i < buf.length && buf[i] == ' ') {
            b.append("&nbsp;") ;
            i++ ;
          }
          i-- ;
          break ;
        }
        default: b.append(buf[i]) ; break ;
      }
    }
    return b.toString();
    
  }
  
  public boolean hasModeratorRole() { return acl_.hasModeratorRole(forum_) ; }

  public boolean hasViewForumRole() { return acl_.hasViewForumRole(forum_) ; }

  public boolean hasCreateTopicRole() { return acl_.hasCreateTopicRole(forum_) ; }

  public boolean hasReplyTopicRole() { return acl_.hasReplyTopicRole(forum_) ; }
  
  public String getUser() { return acl_.getRemoteUser() ; } 

  public UIPageListIterator getUIPageIterator() { return uiPageIterator_ ; }
  
  public boolean canDecodeInvalidState()  { return forum_ != null  || topic_ != null; }  
  
  static public class ViewTopicsActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;			
      UIForumPortlet forumPortlet = 
        (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UITopics.class);
		}
  }
  
  static public class ViewForumsActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;			
      UIForumPortlet forumPortlet = (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UIViewCategories.class);
		}
  }
  
  static public class NewTopicActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;
			UITopics uiTopics = (UITopics) uiPosts.getSibling(UITopics.class) ;
	    Forum forum = uiTopics.getForum() ;
	    UIPostForm uiForm = (UIPostForm) uiPosts.getSibling(UIPostForm.class) ;
	    uiForm.setNewTopicInfo(forum) ;
      UIForumPortlet forumPortlet = (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UIPostForm.class);  
		}
  }
  
  static public class QuotePostActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      String postId = event.getParameter(OBJECTID) ;
      Post post = service.getPost(postId) ;
      UIPostForm uiForm = (UIPostForm) uiPosts.getSibling(UIPostForm.class) ;
      uiForm.setQuotePostInfo(uiPosts.topic_, post) ;
      UIForumPortlet forumPortlet = (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UIPostForm.class);
		}
  }
  
  static public class EditPostActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      String postId = event.getParameter(OBJECTID) ;
      Post post = service.getPost(postId) ;
      UIPostForm uiForm = (UIPostForm) uiPosts.getSibling(UIPostForm.class) ;
      uiForm.setEditPostInfo(uiPosts.topic_, post) ;			      
      UIForumPortlet forumPortlet = (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UIPostForm.class);       
		}
  }
  
  static public class ReplyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIPosts uiPosts = (UIPosts) event.getComponent() ;
			UIPostForm uiForm = (UIPostForm) uiPosts.getSibling(UIPostForm.class) ;
	    uiForm.setNewPostInfo(uiPosts.topic_) ;			
      UIForumPortlet forumPortlet = (UIForumPortlet)uiPosts.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiPosts);      
      forumPortlet.setRenderedComponent(UIPostForm.class);
		}
  }
}