/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.pmanager.component;

import java.util.* ;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.exoplatform.faces.core.component.*;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.pmanager.Project;
import org.exoplatform.services.pmanager.Constant ;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id$
 */
public class UIIssueForm extends UISimpleForm {
  static String SUMMARY = "summary", DESCRIPTION = "description", 
                PRIORITY = "priority", RESOLUTION = "resolution",
                COMPONENTS = "affectedComponent", AFFECTED_VERSION = "affectedVersion",
                ASSIGNEE = "assignee", ISSUE_TYPE = "type", STATUS = "status",
                REPORTER = "reporter", DUE_DATE = "dueDate", 
                TIME_ESTIMATE = "timeEstimate", TIME_SPENT = "timeSpent", 
                TIME_REMAIN = "timeRemain";
  
  static List ISSUE_TYPE_OPTIONS ;
  static List PRIORITY_OPTIONS ;
  static List STATUS_OPTIONS ;
  static {
    ISSUE_TYPE_OPTIONS = new ArrayList(5) ;
    ISSUE_TYPE_OPTIONS.add(new SelectItem("Bug", "bug")) ;
    ISSUE_TYPE_OPTIONS.add(new SelectItem("Feature", "feature")) ;
    ISSUE_TYPE_OPTIONS.add(new SelectItem("Enhancenment", "enhancement")) ;
    
    PRIORITY_OPTIONS = new ArrayList(5) ;
    PRIORITY_OPTIONS.add(new SelectItem("High", "high")) ;
    PRIORITY_OPTIONS.add(new SelectItem("Medium", "medium")) ;
    PRIORITY_OPTIONS.add(new SelectItem("Low", "low")) ;
    
    STATUS_OPTIONS = new ArrayList(5) ;
    STATUS_OPTIONS.add(new SelectItem("Evaluate", "evaluate")) ;
    STATUS_OPTIONS.add(new SelectItem("Open", "open")) ;
    STATUS_OPTIONS.add(new SelectItem("Close", "close")) ;
  }
  
  Project project_ ;
  Node issue_ ;
  Class backComponent_ ;
  
  public UIIssueForm() {
    super("issueForm", "post", null) ;
    setId("UIIssueForm") ;
    setClazz("UIExoForm") ;
    setRendererType("SimpleFormVelocityRenderer") ;
    add(new UIStringInput(SUMMARY, "")) ;
    add(new UISelectBox(ISSUE_TYPE, "bug", ISSUE_TYPE_OPTIONS)) ;
    add(new UISelectBox(PRIORITY, "medium", PRIORITY_OPTIONS)) ;
    add(new UISelectBox(COMPONENTS, "", null).setSize(3)) ;
    add(new UISelectBox(ASSIGNEE, "",null).setSize(3)) ;
    add(new UISelectBox(REPORTER, "",null).setSize(3)) ;
    add(new UISelectBox(AFFECTED_VERSION, "", null).setSize(3)) ;
    add(new UISelectBox(STATUS, "evaluate", STATUS_OPTIONS)) ;
    add(new UITextArea(DESCRIPTION, "")) ;
    add(new UITextArea(RESOLUTION, "")) ;
    Calendar date = new GregorianCalendar() ;
    date.add(Calendar.DAY_OF_YEAR, 60) ;
    add(new UIDateInput(DUE_DATE, date.getTime())) ; 
    add(new UIStringInput(TIME_ESTIMATE, "")) ;
    add(new UIStringInput(TIME_SPENT, "")) ;
    add(new UIStringInput(TIME_REMAIN, "")) ;
   
    addActionListener(SaveActionListener.class, SAVE_ACTION)  ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION)  ;
  }
  
  public void addNewIssue(Project project, Class backComponent) throws Exception {
    issue_ = null ;
    setProject(project, backComponent) ;
    getUISelectBox(ASSIGNEE).setEditable(false);
    getUISelectBox(REPORTER).setEditable(false);
    getUISelectBox(STATUS).setEditable(false);
  }
  
  public void editIssue(Project project, Node issue, Class backComponent) throws Exception {
    issue_ = issue ;
    setProject(project, backComponent) ;
    getUISelectBox(ASSIGNEE).setEditable(true);
    getUISelectBox(REPORTER).setEditable(true);
    getUISelectBox(STATUS).setEditable(true);
    
    setFieldValue(SUMMARY, issue.getProperty(Constant.ISSUE_SUMMARY).getString());
    setFieldValue(COMPONENTS, issue.getProperty(Constant.ISSUE_AFFECTED_COMPONENTS).getString());
    setFieldValue(AFFECTED_VERSION, issue.getProperty(Constant.ISSUE_AFFECTED_VERSION).getString());
    setFieldValue(ASSIGNEE, issue.getProperty(Constant.ISSUE_ASSIGNEE).getString());
    setFieldValue(REPORTER, issue.getProperty(Constant.ISSUE_REPORTER).getString());
    setFieldValue(STATUS, issue.getProperty(Constant.ISSUE_STATUS).getString());
    setFieldValue(DESCRIPTION, issue.getProperty(Constant.ISSUE_DESCRIPTION).getString());
    setFieldValue(RESOLUTION, issue.getProperty(Constant.ISSUE_RESOLUTION).getString());
    setFieldValue(DUE_DATE, issue.getProperty(Constant.ISSUE_DUE_DATE).getDate().getTime());
  }
  
  private void setProject(Project project, Class backComponent) throws Exception {
    project_ = project ;
    backComponent_ = backComponent ;
    reset() ;
    updateComponentSelector(project) ;
    updateVersionSelector(project) ;
    updateAssigneeAdnReporterSelector(project) ;
  }
  
  private void updateComponentSelector(Project project) throws Exception {
    NodeIterator i = project.getComponents() ;
    List list = new ArrayList(5) ;
    list.add(new SelectItem("#{UIIssueForm.label.all-component}", "") ) ;
    while(i.hasNext()) {
      Node node = i.nextNode() ;
      String component = node.getProperty(Constant.PROJECT_COMPONENT_NAME).getString() ;
      list.add(new SelectItem(component, component)) ;
    }
    getUISelectBox(COMPONENTS).setOptions(list) ;
  }
  
  private void updateVersionSelector(Project project) throws Exception {
    NodeIterator i = project.getVersions() ;
    List list = new ArrayList(5) ;
    list.add(new SelectItem("#{UIIssueForm.label.all-version}", "") ) ;
    while(i.hasNext()) {
      Node node = i.nextNode() ;
      String number = node.getProperty(Constant.PROJECT_VERSION_NUMBER).getString() ;
      String name = node.getProperty(Constant.PROJECT_VERSION_NAME).getString() ;
      list.add(new SelectItem(name, number)) ;
    }
    getUISelectBox(AFFECTED_VERSION).setOptions(list) ;
  }
  
  private void updateAssigneeAdnReporterSelector(Project project) throws Exception {
    List list = Util.createParticipantOptions(project) ;
    getUISelectBox(ASSIGNEE).setOptions(list) ;
    getUISelectBox(REPORTER).setOptions(list) ;
  }
  
  static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIIssueForm ui = (UIIssueForm) event.getComponent() ;
      String type = ui.getUISelectBox(ISSUE_TYPE).getValue() ;
      Node node = ui.issue_ ;
      if(node == null) {
        node = ui.project_.createIssue("exo", type) ;
      }
      node.setProperty(Constant.ISSUE_SUMMARY, ui.getUIStringInput(SUMMARY).getValue()) ;
      node.setProperty(Constant.ISSUE_PRIORITY, ui.getUISelectBox(PRIORITY).getValue()) ;
      node.setProperty(Constant.ISSUE_AFFECTED_COMPONENTS, ui.getUISelectBox(COMPONENTS).getValue()) ;
      node.setProperty(Constant.ISSUE_AFFECTED_VERSION, ui.getUISelectBox(AFFECTED_VERSION).getValue()) ;
      node.setProperty(Constant.ISSUE_ASSIGNEE, ui.getUISelectBox(ASSIGNEE).getValue()) ;
      node.setProperty(Constant.ISSUE_REPORTER, ui.getUISelectBox(REPORTER).getValue()) ;
      node.setProperty(Constant.ISSUE_STATUS, ui.getUISelectBox(STATUS).getValue()) ;
      node.setProperty(Constant.ISSUE_DESCRIPTION, ui.getUIStringInput(DESCRIPTION).getValue()) ;
      node.setProperty(Constant.ISSUE_RESOLUTION, ui.getUIStringInput(RESOLUTION).getValue()) ;
      node.setProperty(Constant.ISSUE_DUE_DATE, ui.getUIDateInput(DUE_DATE).getCalendar()) ;
      ui.project_.save() ;
      ui.setRenderedSibling(ui.backComponent_);
		}
	}
  
  static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIIssueForm uiForm = (UIIssueForm) event.getComponent() ;
      uiForm.setRenderedSibling(uiForm.backComponent_);
		}
	}
}