/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.hibernate;

import java.text.MessageFormat;
import java.util.List;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumEventListener;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.Watcher;
import org.exoplatform.services.mail.MailService;
import org.exoplatform.services.mail.Message;
import org.exoplatform.services.scheduler.JobSchedulerService;
import org.exoplatform.services.scheduler.Task;
import org.hibernate.Session;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 25, 2004
 * @version $Id$
 */
public class WatcherNotifyListener extends ForumEventListener {
  private String  forumChangeMessage_ ;
  private String  forumChangeSubject_ ;
  private String topicChangeMessage_ ;
  private String topicChangeSubject_ ;
  private String messageAuthor_ ;  
  private String messageReceiver_ ;
  
  private MailService mservice_ ;
  private JobSchedulerService jservice_ ;
  private HibernateService hservice_ ;    
  
  public WatcherNotifyListener(MailService mservice,
                               InitParams params,
                               HibernateService hservice,
                               JobSchedulerService jservice) throws Exception {
    mservice_  = mservice ;
    jservice_ = jservice ;
    hservice_ = hservice ;
    forumChangeSubject_ = params.getValueParam("forum.change.subject").getValue() ;
    forumChangeMessage_ = params.getValueParam("forum.change.message").getValue() ;
    topicChangeSubject_ = params.getValueParam("topic.change.subject").getValue() ;
    topicChangeMessage_ = params.getValueParam("topic.change.message").getValue() ;
    messageAuthor_ = params.getValueParam("message.author").getValue() ;    
  }
  
  public void onSave(Forum forum, Topic topic, Post post) throws Exception  {    
    Session session = hservice_.openSession()  ;
    PostImpl impl = (PostImpl) post ; 
    ObjectQuery q = new ObjectQuery(WatcherImpl.class).
    addEQ("target",  WatcherImpl.FORUM_TARGET).
    addEQ("forumId", impl.getForumId()) ;
    List<Watcher> watchers = session.createQuery(q.getHibernateQuery()).list();    
    //.find(q.getHibernateQuery()) ;        
    if(watchers.size() == 0) return ;
    Object[] params = {forum.getForumName(), post.getSubject(), post.getMessage()} ;
    String subject = MessageFormat.format(forumChangeSubject_, params) ;
    String body  = MessageFormat.format(forumChangeMessage_, params) ;    
    send(watchers, subject, body) ;    
    q = new ObjectQuery(WatcherImpl.class).
        addEQ("target",  WatcherImpl.TOPIC_TARGET).
        addEQ("topicId", impl.getTopicId()) ;
    watchers = session.createQuery(q.getHibernateQuery()).list();
  //.find(q.getHibernateQuery()) ;    
    if(watchers.size() == 0) return ;    
    Object[] topicparams = {topic.getTopic(), post.getSubject(), post.getMessage()} ;
    subject = MessageFormat.format(topicChangeSubject_, topicparams) ;
    body  = MessageFormat.format(topicChangeMessage_, topicparams) ;
    for(Watcher w:watchers) {                  
    }
    send(watchers, subject, body) ;    
  }
  
 
  private void send(List watchers, String subject, String body) throws Exception {
    StringBuffer bcc = new StringBuffer() ;
    Watcher watcher = (Watcher) watchers.get(0) ;
    messageReceiver_ = watcher.getAddress() ;    
    for(int i = 1; i < watchers.size(); i++) {
      if(i > 1) bcc.append(", ") ;
      watcher = (Watcher) watchers.get(i) ;
      bcc.append(watcher.getAddress()) ;
    }           
    Message message = new Message() ;    
    message.setFrom(messageAuthor_) ;
    message.setTo(messageReceiver_) ;
    message.setBCC(bcc.toString()) ;
    message.setSubject(subject) ;
    message.setBody(body) ;        
    jservice_.queueTask(new NotifyForumWatcherTask(mservice_,message)) ;    
  }
  
  static public class NotifyForumWatcherTask extends Task{
    private MailService mservice_ ;
    private Message message_ ;
    
    public NotifyForumWatcherTask(MailService service, Message message) {
      mservice_ = service ;
      message_ = message ;
    }
    
    public void execute() throws Exception {      
      mservice_.sendMessage(message_) ;
    }

    public String getName() { return "NotifyForumWatcherTask" ;}
    public String getDescription() { return "Send message : " + message_.getSubject(); }
  }
}