/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.datamodel;

import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: NodeData.java 3510 2005-12-29 17:32:57Z geaz $
 */

public interface NodeData extends ItemData {
	
	/**
	 * @return ordered by orderNum set of child nodes' NodeData. NodeData should support lazy 
	 * initialization of its children. It means that it is possible that NodeData
	 * does not contain any children before calling getNodes()/getProperties() 
	 * @throws IllegalStateException if child nodes could not be initialized
	 */
	List getChildNodes() throws IllegalStateException, RepositoryException;

	/**
	 * @return set of child properties' PropertyData. The same lazy initialization policy as for
	 * child nodes
	 * @throws IllegalStateException
	 */
	List getChildProperties() throws IllegalStateException, RepositoryException;

	
	/**
	 * @return this node order number 
	 */
	int getOrderNumber();

	/**
	 * @return name of primary node type of this node. The jcr:primaryType property
	 * is loaded twice here and lazy initialized like others.
	 */
	InternalQName getPrimaryTypeName();
	
	/**
	 * @return names of mixin node types. The jcr:mixinTypes property
	 * is loaded twice here and lazy initialized like others.
	 * return empty array if no mixin types found
	 */
	InternalQName[] getMixinTypeNames();
	

	/**
	 * @return access control list for children's permission
	 */
	AccessControlList getChildrenACL();
	

}