/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl;

import java.io.IOException;

import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NodeTypeManager;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.jmx.MX4JComponentAdapterFactory;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.core.BatcheableWorkspaceDataManager;
import org.exoplatform.services.jcr.impl.core.NamespaceRegistryImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.UUIDGenerator;
import org.exoplatform.services.jcr.impl.core.WorkspaceDataManager;
import org.exoplatform.services.jcr.impl.core.access.AccessManager;
import org.exoplatform.services.jcr.impl.core.lock.LockManager;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.core.observation.ObservationManagerRegistry;
import org.exoplatform.services.jcr.impl.core.query.QueryHandler;
import org.exoplatform.services.jcr.impl.core.query.QueryHandlerContext;
import org.exoplatform.services.jcr.impl.core.query.QueryManagerFactory;
import org.exoplatform.services.jcr.impl.core.query.SearchManager;
import org.exoplatform.services.jcr.impl.core.version.VersionStorageManager;
import org.exoplatform.services.jcr.impl.storage.cache.WorkspaceCache;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: WorkspaceContainer.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class WorkspaceContainer extends ExoContainer {

  private String name;

  private RepositoryContainer repositoryContainer;
  
  private MBeanServer mbeanServer;


  public WorkspaceContainer(
      RepositoryContainer parent, 
      WorkspaceEntry config)
      throws RepositoryException, RepositoryConfigurationException {

    //Before repository instantiation
    //super(parent);
    super(new MX4JComponentAdapterFactory(), parent);

    repositoryContainer = parent;
    this.name = config.getName();
    
    this.mbeanServer = MBeanServerFactory.createMBeanServer("jcrws" + name + "at"
        + repositoryContainer.getName() + "mx");

    config.setUniqueName(parent.getName()+"_"+name);
    
    registerComponentInstance(config);

    try {
      registerComponentImplementation(Class.forName(config.getContainer()
          .getType()));
    } catch (ClassNotFoundException e) {
      throw new RepositoryConfigurationException(
          "Class not found for workspace data container: " + e);
    }

    registerComponentImplementation(WorkspaceCache.class);

    registerComponentImplementation(BatcheableWorkspaceDataManager.class);

    registerComponentImplementation(ObservationManagerRegistry.class);
    
    registerComponentImplementation(VersionStorageManager.class);
    
    registerComponentImplementation(LockManager.class);

    
    if (config.getQueryHandler() != null) {
      try {

        Class qh = Class.forName(config.getQueryHandler().getType());
        registerComponentImplementation(qh);
      } catch (ClassNotFoundException e) {
        throw new RepositoryConfigurationException(
            "Class not found for workspace query handler " + e);
      }

      registerComponentImplementation(SearchManager.class);
      registerComponentImplementation(QueryManagerFactory.class);
    }

  }

  public MBeanServer getMBeanServer() {
    return mbeanServer;
  }

  public void init(WorkspaceEntry config, RepositoryImpl repository,
      boolean system) throws RepositoryException,
      RepositoryConfigurationException {

    //After repository instantiation
    SessionImpl sysSession = new SessionImpl(name, AccessManager
        .systemCredentials(), this);
    registerComponentInstance(sysSession);
    
    ((BatcheableWorkspaceDataManager) getComponentInstanceOfType(BatcheableWorkspaceDataManager.class))
    .setSystemSession(repository.getSystemSession());

    
    if (system) {

      repositoryContainer.registerComponentInstance(sysSession);
      
      // Set the Session
      ((NamespaceRegistryImpl) getComponentInstanceOfType(NamespaceRegistry.class))
          .setSystemSession(sysSession);
      ((NodeTypeManagerImpl) getComponentInstanceOfType(NodeTypeManager.class))
          .setSystemSession(sysSession);
      

      ((NamespaceRegistryImpl) getComponentInstanceOfType(NamespaceRegistry.class))
          .loadFromStorage();

      ((NodeTypeManagerImpl) getComponentInstanceOfType(NodeTypeManager.class))
          .loadFromStorage();

    }
    
    //((VersionStorageManager) getComponentInstanceOfType(VersionStorageManager.class))
    //.setSystemSession(sysSession);
    


    
    // Init Root 
    if (config.getAutoInitializedRootNt() != null
        && !repository.isWorkspaceInitialized(name))
      repository.initWorkspace(name, config
          .getAutoInitializedRootNt());
    
    // ROOT node shoould be initialized already
    if (repository.isWorkspaceInitialized(name)) {
      
      WorkspaceDataManager dataManager = (WorkspaceDataManager)
        getComponentInstanceOfType(WorkspaceDataManager.class);

      QueryHandlerContext context = new QueryHandlerContext(
          dataManager, UUIDGenerator.ROOT_UUID, sysSession
          .getLocationFactory());


      QueryHandler handler = (QueryHandler) getComponentInstanceOfType(QueryHandler.class);
      if (handler != null) {
        try {
          handler.init(context);
        } catch (IOException e) {
          e.printStackTrace();
          throw new RepositoryException(e);
        }
      }
    }
    
    
  }

}