/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.jcr.AccessDeniedException;
import javax.jcr.Credentials;
import javax.jcr.ImportUUIDBehavior;
import javax.jcr.InvalidItemStateException;
import javax.jcr.InvalidSerializedDataException;
import javax.jcr.Item;
import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.LoginException;
import javax.jcr.NamespaceException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.ValueFormatException;
import javax.jcr.Workspace;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.observation.ObservationManager;
import javax.jcr.version.VersionException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.codec.binary.Base64;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.jcr.core.NamespaceAccessor;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.access.AccessManager;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeCopyVisitor;
import org.exoplatform.services.jcr.impl.core.lock.LockManager;
import org.exoplatform.services.jcr.impl.core.observation.ObservationManagerRegistry;
import org.exoplatform.services.jcr.impl.core.query.QueryManagerFactory;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.jcr.impl.util.NodeTypeRecognizer;
import org.exoplatform.services.jcr.impl.util.StringConverter;
import org.exoplatform.services.jcr.impl.xml.DocNodeImporter;
import org.exoplatform.services.jcr.impl.xml.ExporterSysView;
import org.exoplatform.services.jcr.impl.xml.SysNodeImporter;
import org.exoplatform.services.jcr.impl.xml.XMLWriter;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.impl.ExoLog;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: SessionImpl.java 3607 2006-01-04 11:28:33Z peterit $ The
 *          implementation supported CredentialsImpl
 */
public class SessionImpl implements Session, NamespaceAccessor {

  protected ExoLog            log;

  private RepositoryImpl      repository;

  private CredentialsImpl     credentials;

  private WorkspaceImpl       workspace;

  private SessionDataManager  nodesManager;

  private Map                 namespaces;

  private AccessManager       accessManager;

  private LocationFactory     locationFactory;

  private ValueFactory        valueFactory;

  private QueryManagerFactory queryManagerFactory;

  private ExoContainer        container;

  // private NodeTypeValidator nodeTypeValidator;
  private LogService          logService;

  private LocationFactory     systemLocationFactory;

  private UUIDGenerator       uuidGenerator;

  private LockManager         lockManager;

  private String              workspaceName;

  private boolean             live;

  private HashSet             lockTokens;

  private List                lifecycleListeners;

  public SessionImpl(String workspaceName, Credentials credentials, ExoContainer container)
      throws RepositoryException {

    this.workspaceName = workspaceName;
    this.container = container;
    this.live = true;
    this.lockTokens = new HashSet();
    this.logService = (LogService) container.getComponentInstanceOfType(LogService.class);

    // this.log = logService.getLog("org.exoplatform.services.jcr");
    this.log = (ExoLog) logService.getLog(this.getClass());
    log.setLevel(LogService.DEBUG);

    this.repository = (RepositoryImpl) container.getComponentInstanceOfType(RepositoryImpl.class);
    this.systemLocationFactory = (LocationFactory) container
        .getComponentInstanceOfType(LocationFactory.class);
    this.uuidGenerator = (UUIDGenerator) container.getComponentInstanceOfType(UUIDGenerator.class);
    this.accessManager = (AccessManager) container.getComponentInstanceOfType(AccessManager.class);
    this.lockManager = (LockManager) container.getComponentInstanceOfType(LockManager.class);

    this.credentials = (CredentialsImpl) credentials;
    this.locationFactory = new LocationFactory(this);
    this.valueFactory = new ValueFactoryImpl(locationFactory);
    this.namespaces = new HashMap();

    // Observation manager per session
    ObservationManager observationManager = ((ObservationManagerRegistry) container
        .getComponentInstanceOfType(ObservationManagerRegistry.class))
        .createObservationManager(this);
    // ObservationManager observationManager = (ObservationManager) container
    // .getComponentInstanceOfType(ObservationManager.class);

    this.workspace = new WorkspaceImpl(workspaceName, container, this, observationManager);

    BatcheableWorkspaceDataManager workspaceDataManager = (BatcheableWorkspaceDataManager) container
        .getComponentInstanceOfType(BatcheableWorkspaceDataManager.class);

    this.nodesManager = new SessionDataManager(this, workspaceDataManager, logService);
    // this.nodesManager = new SessionDataManager();

    this.lifecycleListeners = new ArrayList();
    this.registerLifecycleListener(lockManager);

  }

  /** @see javax.jcr.Session#getUserId */
  public String getUserID() {
    return credentials.getUserID();
  }

  /** @see javax.jcr.Session#getUserId */
  public Object getAttribute(String name) {
    return credentials.getAttribute(name);
  }

  /** @see javax.jcr.Session#getAttributeNames */
  public String[] getAttributeNames() {
    return credentials.getAttributeNames();
  }

  /** @see javax.jcr.Session#logout */
  public void logout() {
    for (int i = 0; i < lifecycleListeners.size(); i++)
      ((SessionLifecycleListener) lifecycleListeners.get(i)).onCloseSession(this);
    live = false;
  }

  /** @see javax.jcr.Session#isLive */
  public boolean isLive() {
    return live;
  }

  /** @see javax.jcr.Session#getRepository */
  public Repository getRepository() {
    return repository;
  }

  /** @see javax.jcr.Session#impersonate */
  public Session impersonate(Credentials credentials) throws LoginException, RepositoryException {
    if (credentials instanceof CredentialsImpl)
      return new SessionImpl(workspaceName, credentials, container);
    else if (credentials instanceof SimpleCredentials) {
      String name = ((SimpleCredentials) credentials).getUserID();
      char[] pswd = ((SimpleCredentials) credentials).getPassword();
      CredentialsImpl thisCredentials = new CredentialsImpl(name, pswd);
      return new SessionImpl(workspaceName, thisCredentials, container);
    } else
      throw new LoginException(
          "Credentials for the authentication should be CredentialsImpl or SimpleCredentials type");
  }

  /** @see javax.jcr.Session#getWorkspace */
  public Workspace getWorkspace() {
    return workspace;
  }

  /** @see javax.jcr.Session#getRootNode */
  public Node getRootNode() throws RepositoryException {
    return (Node) getItem(JCRPath.ROOT_PATH);
  }

  /**
   * @see javax.jcr.Session#getNodeByUUID
   */
  public Node getNodeByUUID(String uuid) throws ItemNotFoundException, RepositoryException {
    Item item = nodesManager.getItemByUUID(uuid);

    if (item != null && item.isNode()) {
      Node node = (Node) item;
      log.debug("Get node by UUID " + node);
      node.getUUID(); // throws exception
      return node;
    } else
      throw new ItemNotFoundException("Node not found " + uuid + " at " + workspaceName);

  }

  /** @see javax.jcr.Session#getItem */
  public Item getItem(String absPath) throws PathNotFoundException, RepositoryException {
    JCRPath loc = locationFactory.parseAbsPath(absPath);
    log.debug("Session.getItem() " + loc.getAsString(false));
    ItemImpl item = nodesManager.getItem(loc);
    if (item != null)
      return item;
    else
      throw new PathNotFoundException("Item not found " + absPath + " in workspace "
          + workspaceName);
  }

  /** @see javax.jcr.Session#itemExists */
  public boolean itemExists(String absPath) {
    try {
      if (nodesManager.getItem(locationFactory.parseAbsPath(absPath)) != null)
        return true;
    } catch (RepositoryException e) {
    }
    return false;
  }

  /** @see javax.jcr.Session#save */
  public void save() throws AccessDeniedException, LockException, ConstraintViolationException,
      InvalidItemStateException, RepositoryException {
    getRootNode().save();
  }

  /** @see javax.jcr.Session#refresh refresh(false) does not work */
  public void refresh(boolean keepChanges) throws RepositoryException {
    getRootNode().refresh(keepChanges);
  }

  /** @see javax.jcr.Session#hasPendingChanges */
  public boolean hasPendingChanges() throws RepositoryException {
    return nodesManager.hasPendingChanges(locationFactory.createRootLocation(), false);
  }

  /** @see javax.jcr.Session#getValueFactory */
  public ValueFactory getValueFactory() throws UnsupportedRepositoryOperationException,
      RepositoryException {
    return valueFactory;
  }

  /** @see javax.jcr.Session#getNamespacePrefix */
  public String getNamespacePrefix(String uri) throws NamespaceException, RepositoryException {
    String[] prefixes = getNamespacePrefixes();
    for (int i = 0; i < prefixes.length; i++) {
      if (getNamespaceURI(prefixes[i]).equals(uri))
        return prefixes[i];
    }
    throw new NamespaceException("Prefix for " + uri + " not found");
  }

  /** @see javax.jcr.Session#setNamespacePrefix */
  public void setNamespacePrefix(String prefix, String uri) throws NamespaceException,
      RepositoryException {
    ((NamespaceRegistryImpl) workspace.getNamespaceRegistry()).validateNamespace(prefix, uri);
    String testPrefix = workspace.getNamespaceRegistry().getPrefix(uri);
    if (testPrefix.equals(prefix)) // no needs to remap
      return;
    try {
      workspace.getNamespaceRegistry().getURI(prefix);
    } catch (NamespaceException e) {
      namespaces.put(prefix, uri);
      return;
    }
    throw new NamespaceException("Prefix " + prefix + " is already mapped to other uri");
  }

  /** @see javax.jcr.Session#getNamespacePrefixes */
  public String[] getNamespacePrefixes() throws RepositoryException {
    Collection allPrefixes = new LinkedList();
    allPrefixes.addAll(namespaces.keySet());
    String[] permanentPrefixes = workspace.getNamespaceRegistry().getPrefixes();
    for (int i = 0; i < permanentPrefixes.length; i++) {
      // log.info("PREFIX Session.getnamespaceprefixes "+permanentPrefixes[i]);
      String permanentPrefix = permanentPrefixes[i];
      String uri = null;
      try {
        uri = workspace.getNamespaceRegistry().getURI(permanentPrefix);
        if (!allPrefixes.contains(permanentPrefix) && !(namespaces.values().contains(uri))) {
          allPrefixes.add(permanentPrefix);
        }
      } catch (RepositoryException e) {
        e.printStackTrace();
      }
    }
    return (String[]) allPrefixes.toArray(new String[allPrefixes.size()]);
  }

  /** @see javax.jcr.Session#getNamespaceURI */
  public String getNamespaceURI(String prefix) throws NamespaceException, RepositoryException {
    String uri = null;
    // look in session first
    uri = (String) namespaces.get(prefix);
    if (uri != null)
      return uri;
    uri = workspace.getNamespaceRegistry().getURI(prefix);
    if (namespaces.values().contains(uri))
      return null;
    if (uri == null)
      throw new NamespaceException("No namespace '" + uri + "' found");
    return uri;
  }

  public String getNamespaceURIByPrefix(String prefix) throws NamespaceException,
      RepositoryException {
    return getNamespaceURI(prefix);
  }

  public String getNamespacePrefixByURI(String uri) throws NamespaceException, RepositoryException {
    return getNamespacePrefix(uri);
  }

  public String[] getAllNamespacePrefixes() throws RepositoryException {
    return getNamespacePrefixes();
  }

  /** @see javax.jcr.Session#exportSysView */
  public void exportSystemView(String absPath, ContentHandler contentHandler, boolean skipBinary,
      boolean noRecurse) throws PathNotFoundException, SAXException, RepositoryException {

    ExporterSysView exporter = new ExporterSysView(this);
    exporter.setContentHandler(contentHandler);
    exporter.setRecurse(!noRecurse);
    exporter.setBinaryConduct(skipBinary ? ExporterSysView.BINARY_SKIP
        : ExporterSysView.BINARY_PROCESS);

    JCRPath loc = locationFactory.parseAbsPath(absPath);
    NodeImpl node = (NodeImpl) nodesManager.getItem(loc);
    exporter.export(node);

  }

  /**
   * @see javax.jcr.Session#exportSysView
   * @todo check namespace definition correct output
   */
  public void exportSystemView(String absPath, OutputStream out, boolean skipBinary,
      boolean noRecurse) throws IOException, PathNotFoundException, RepositoryException {
    TransformerHandler handler;
    try {
      SAXTransformerFactory saxFact = (SAXTransformerFactory) SAXTransformerFactory.newInstance();
      handler = saxFact.newTransformerHandler();
      handler.setResult(new StreamResult(out));

      // need to solve trouble with XMLQuery
      // handler.getTransformer().setOutputProperty(
      // OutputKeys.OMIT_XML_DECLARATION, "yes");
    } catch (javax.xml.transform.TransformerFactoryConfigurationError ex) {
      throw new IOException(ex.getMessage());
    } catch (javax.xml.transform.TransformerConfigurationException ex) {
      throw new IOException(ex.getMessage());
    }

    try {
      exportSystemView(absPath, handler, skipBinary, noRecurse);
    } catch (SAXException ex) {
      throw new RepositoryException("Error on export", ex);
    }
  }

  /** @see javax.jcr.Session#exportDocView */
  public void exportDocumentView(String absPath, ContentHandler contentHandler, boolean skipBinary,
      boolean noRecurse) throws InvalidSerializedDataException, PathNotFoundException,
      SAXException, RepositoryException {
    NodeImpl node = (NodeImpl) nodesManager.getItem(locationFactory.parseAbsPath(absPath));
    XMLWriter writer = new XMLWriter(this);

    initNodeAsDocView(node, writer, skipBinary, noRecurse);

    invokeHandler(writer.getBytes(), contentHandler);
  }

  /** @see javax.jcr.Session#exportDocView */
  public void exportDocumentView(String absPath, OutputStream out, boolean skipBinary,
      boolean noRecurse) throws InvalidSerializedDataException, IOException, PathNotFoundException,
      RepositoryException {
    NodeImpl node = (NodeImpl) nodesManager.getItem(locationFactory.parseAbsPath(absPath));

    XMLWriter writer = new XMLWriter(this);
    initNodeAsDocView(node, writer, skipBinary, noRecurse);
    out.write(writer.getBytes());
    out.flush();
  }

  /**
   * @param uuidBehavior TODO
   * @see javax.jcr.Session#importXML
   */
  public void importXML(String parentAbsPath, InputStream in, int uuidBehavior) throws IOException,
      PathNotFoundException, ItemExistsException, ConstraintViolationException,
      InvalidSerializedDataException, RepositoryException {

    int type;
    byte[] buffer = new byte[in.available()];
    ByteArrayInputStream str;
    try {
      in.read(buffer);
      str = new ByteArrayInputStream(buffer);
      type = NodeTypeRecognizer.recognize(str);
    } catch (SAXException e) {
      throw new InvalidSerializedDataException(e.getMessage());
    } catch (Exception e) {
      // throw new IOException(e.getMessage());
      throw new InvalidSerializedDataException(e.getMessage());
    }
    str = new ByteArrayInputStream(buffer);

    if (type == NodeTypeRecognizer.SYS) {
      importSysView(parentAbsPath, str);
    } else {
      importDocView(parentAbsPath, str, uuidBehavior);
    }
  }

  /**
   * @param uuidBehavior TODO
   * @see javax.jcr.Session#getImportContentHandler add default rule
   */
  public ContentHandler getImportContentHandler(String parentAbsPath, int uuidBehavior)
      throws PathNotFoundException, ConstraintViolationException, VersionException,
      RepositoryException {

    NodeImpl node = (NodeImpl) getItem(parentAbsPath);
    
    // checked-in
    if (!node.isCheckedOut()) {
      throw new VersionException("Node " + node.getPath() + " or its nearest ancestor is checked-in");
    }
    
    // Check if node is not protected
    if (node.getDefinition().isProtected()) {
      throw new ConstraintViolationException("Can't add protected node "
          + node.getName() + " to " + node.getParent().getPath());
    }

    // Check locking
    if (!node.checkLocking()) {
      throw new LockException("Node " + node.getPath() + " is locked ");
    }
    
    DocNodeImporter handler = new DocNodeImporter(node);

    handler.setUuidBehavior(uuidBehavior);

    return handler;
  }

  /** @see javax.jcr.Session#move */
  public void move(String srcAbsPath, String destAbsPath) throws ItemExistsException,
      PathNotFoundException, VersionException, RepositoryException {

    NodeImpl srcNode = (NodeImpl) this.getItem(srcAbsPath);

    JCRPath destNodePath = getLocationFactory().parseAbsPath(destAbsPath);
    if (destNodePath.isIndexSetExplicitly())
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + destNodePath.getAsString(false));

    String parentPath = destNodePath.makeParentPath().getAsString(false);
    NodeImpl parentNode = (NodeImpl) getItem(parentPath);

    NodeImpl destNode = parentNode.createChildNode(destNodePath.getName().getAsString(), srcNode
        .getPrimaryNodeType().getName(), srcNode.getMixinTypeNames(), true);

    NodeCopyVisitor initializer = new NodeCopyVisitor(destNode, false);
    srcNode.accept(initializer);

    // NodeImpl destNode = (NodeImpl) parentNode.getNode(srcNode.getName());
    destNode.setState(ItemChangeState.ADDED);
    nodesManager.update(destNode);
    srcNode.remove();
  }

  // //////////////////// OPTIONAL

  /**
   * @see javax.jcr.Session#checkPermission
   */
  public void checkPermission(String absPath, String actions) throws AccessControlException {

    try {
      JCRPath parentPath = locationFactory.parseAbsPath(absPath).makeParentPath();
      NodeImpl parent = (NodeImpl) nodesManager.getItem(parentPath);
      if (parent == null) // root
        return;
      if (!accessManager.hasPermission(parent.getChildrenACL(), actions, getUserID()))
        throw new AccessControlException("Permission denied " + absPath + " : " + actions);
    } catch (RepositoryException e) {
      throw new AccessControlException("Can not check permission for " + absPath + " " + e);
    }
  }

  /** @see javax.jcr.Session#addLockToken */
  public void addLockToken(String lt) {
    lockTokens.add(lt);
  }

  /** @see javax.jcr.Session#getLockTokens */
  public String[] getLockTokens() {
    String[] tokens = new String[lockTokens.size()];
    lockTokens.toArray(tokens);
    return tokens;
  }

  /** @see javax.jcr.Session#removeLockToken */
  public void removeLockToken(String lt) {
    lockTokens.remove(lt);
  }

  // //////////////////// IMPL

  public String getId() {
    return toString();
  }

  public SessionDataManager getTransientNodesManager() {
    return this.nodesManager;
  }

  Credentials getCredentials() {
    return this.credentials;
  }

  LocationFactory getSystemLocationFactory() {
    return systemLocationFactory;
  }

  LockManager getLockManager() {
    return lockManager;
  }

  void registerLifecycleListener(SessionLifecycleListener listener) {
    this.lifecycleListeners.add(listener);
  }

  boolean hasLockToken(String lt) {
    return lockTokens.contains(lt);
  }

  public String getLockToken(String lt) {
    for (Iterator it = lockTokens.iterator(); it.hasNext();) {
      String token = (String) it.next();
      if (token.equals(lt))
        return token;
    }
    return null;
  }

  // ////////////////////// PRIVATE
  private void importSysView(String parentPath, InputStream in) throws PathNotFoundException,
      InvalidSerializedDataException, ConstraintViolationException, RepositoryException {
    ArrayList items;
    try {
      SysNodeImporter importer = new SysNodeImporter((NodeImpl) getItem(parentPath));
      NodeImpl node = importer.parse(in);
    } catch (IOException e) {
      throw new InvalidSerializedDataException(e.getMessage(), e);
    } catch (SAXException e) {
      throw new InvalidSerializedDataException(e.getMessage(), e);
    }
  }

  private void importDocView(String parentPath, InputStream in, int uuidBehavior)
      throws PathNotFoundException, InvalidSerializedDataException, ConstraintViolationException,
      RepositoryException {

    try {
      // DocNodeImporter importer = new DocNodeImporter((NodeImpl)
      // getItem(parentPath));
      // importer.setUuidBehavior(uuidBehavior);

      DocNodeImporter importer = (DocNodeImporter) getImportContentHandler(parentPath, uuidBehavior);

      NodeImpl node = importer.parse(in);
      node.setState(ItemChangeState.ADDED);
      nodesManager.update(node);
    } catch (IOException e) {
      e.printStackTrace();
      throw new InvalidSerializedDataException("importDocView failed", e);
    } catch (SAXException e) {
      e.printStackTrace();
      // throw new InvalidSerializedDataException("importDocView failed", e);
      if (uuidBehavior == ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW) {
        throw new ItemExistsException("importDocView failed", e);
      } else {
        throw new ConstraintViolationException("importDocView failed", e);
      }
    }

  }

  // Helper for for node export
  private void initNodeAsDocView(NodeImpl node, XMLWriter writer, boolean skipBinary,
      boolean noRecurse) throws RepositoryException {

    String name = node.getName();
    if (name.length() == 0) // root node
      name = "jcr:root";

    Properties attrs = new Properties();

    PropertyIterator props = node.getProperties();
    while (props.hasNext()) {
      PropertyImpl prop = (PropertyImpl) props.next();

      String[] strPropValues = getStrPropValues(prop, skipBinary);
      if (strPropValues == null || strPropValues.length == 0) // skip
        continue;
      String strValues = "";
      for (int i = 0; i < strPropValues.length; i++) {
        strValues += strPropValues[i];
        if (i < strPropValues.length - 1 && strPropValues[i].length() > 0)
          // space as delimiter for multi-valued (not applied for skipBinary)
          strValues += " ";
        // System.out.println(">>> PROP >>>> "+prop.getPath()+" "+strValues);
      }
      attrs.setProperty(prop.getName(), strValues);
    }
    writer.startElement(node.getLocation().getName(), attrs);

    NodeIterator nodes = node.getNodes();
    while (nodes.hasNext()) {
      NodeImpl child = (NodeImpl) nodes.nextNode();
      if (!noRecurse) {
        if (child.getLocation().getName().getInternalName().equals(Constants.JCR_XMLTEXT)) {
          try {
            // TODO jcr:xmlcharacters
            String val = StringConverter.normalizeString(child.getProperty("jcr:xmlcharacters")
                .getString(), false);
            // System.out.println(">>> TEXT >>>> "+child.getPath()+" "+val);
            writer.writeText(val);
            continue;
          } catch (ValueFormatException e) {
            // will init as element
          } catch (PathNotFoundException e) {
            // will init as element
          }
        }
        initNodeAsDocView(child, writer, skipBinary, noRecurse);
      }

    }

    writer.endElement();
  }

  private void invokeHandler(byte[] input, ContentHandler contentHandler) throws SAXException,
      RepositoryException {
    try {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);
      // System.out.println(" factory.isNamespaceAware() "
      // + factory.isNamespaceAware());
      SAXParser parser = factory.newSAXParser();

      XMLReader reader = parser.getXMLReader();

      reader.setContentHandler(contentHandler);

      System.out.println(" Session.invokeHandler: " + new String(input));
      // saveAsFile(input, "d:/tmp/export_test_sys.xml");

      reader.parse(new InputSource(new ByteArrayInputStream(input)));

    } catch (SAXException e) {
      e.printStackTrace();
      throw e;
    } catch (Exception e) {
      e.printStackTrace();
      throw new RepositoryException("Can not invoke content handler", e);
    }
  }

  public void _exportRepositoryDebug(String sysPath) {
    FileOutputStream out;
    try {
      
      ByteArrayOutputStream fis = new ByteArrayOutputStream();      
      exportSystemView(sysPath, fis, false, false);
      byte[] repo = fis.toByteArray();
      
      System.err.println("======== repository: " + new String(repo));
      
      //saveAsFile(repo, "D:/tmp/export1.xml");
                   
    } catch (Exception e) {
      System.err.println("======== Could not export the repository: " + e);
    }
  }  
  
  public void _saveRepoPath(String repoPath) {
    // For debug, save the repo path
    try {
      System.err.println("========= Save path in repo: " + repoPath);
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      exportDocumentView(repoPath, baos, false, false);
      System.err.println("========= Saved xml: >>> [" + new String(baos.toByteArray()) + "] <<<");
      //saveAsFile(baos.toByteArray(), "d:/tmp/repo-" + System.currentTimeMillis() + ".xml");
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("========= Save xml: Error: " + e);
    }
  }

  public void _saveAsFile(byte[] data, String fileName) {
    try {
      FileOutputStream fos = new FileOutputStream(fileName);
      fos.write(data);
      fos.flush();
      fos.close();
    } catch (Exception e) {
      System.out.println("========  DEBUG: Error save as file " + fileName + ". Error:" + e);
    }
  }

  private byte[] readBinaryValue(BinaryValue binVal) throws ValueFormatException,
      RepositoryException {

    InputStream is = binVal.getStream();
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    byte[] buff = new byte[8192];
    try {
      while (is.available() > 0) {
        int r = is.read(buff);
        baos.write(buff, 0, r);
      }
      return baos.toByteArray();
    } catch (IOException e) {
      throw new ValueFormatException("read from BinaryValue stream to byte array failed", e);
    }
  }

  private String[] getStrPropValues(PropertyImpl prop, boolean skipBinary)
      throws ValueFormatException, RepositoryException {

    String[] values = new String[prop.getValueArray().length];
    if (values.length == 0)
      return null;

    if (prop.getType() == PropertyType.BINARY) {
      // if (skipBinary)
      // values = null;
      // else {
      if (skipBinary) {
        for (int i = 0; i < values.length; i++)
          values[i] = "";
      } else {
        for (int i = 0; i < values.length; i++) {
          byte[] buff = readBinaryValue((BinaryValue) prop.getValueArray()[i]);
          values[i] = new String(Base64.encodeBase64(buff));
        }
      }
      // }
    } else {
      for (int i = 0; i < values.length; i++) {
        Value val = prop.getValueArray()[i];
        try {
          if (val != null)
            values[i] = StringConverter.normalizeString(val.getString(), true);
        } catch (ValueFormatException e) {
          if (!e.getMessage().equals("empty value")) {
            throw e;
          }
        }
      }
    }
    return values;
  }

  /**
   * @return Returns the locationFactory.
   */
  public LocationFactory getLocationFactory() {
    return locationFactory;
  }

  /**
   * @return Returns the uuidGenerator.
   */
  public UUIDGenerator getUuidGenerator() {
    return uuidGenerator;
  }

  /**
   * @return Returns the accessManager.
   */
  public AccessManager getAccessManager() {
    return accessManager;
  }
}