/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.Calendar;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.BatcheableWorkspaceDataManager;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeDumpVisitor;
import org.exoplatform.services.jcr.impl.core.value.NameValue;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: VersionImpl.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class VersionImpl extends VersionStorageDescendantNode implements Version {

  public VersionImpl(NodeData data, NodeImpl parent, int state,
      SessionImpl session, LogService logService) throws PathNotFoundException,
      RepositoryException {

    super(data, parent, state, session, logService);

    if (!this.getPrimaryNodeType().isNodeType("nt:version"))
      throw new RepositoryException("Node " + getLocation().getAsString(true)
          + " is not nt:version type");

  }

  public VersionHistory getContainingHistory() throws RepositoryException {
    return (VersionHistory) getParent();
  }

  public Calendar getCreated() throws RepositoryException {
    return getProperty("jcr:created").getDate();
  }

  public Version[] getSuccessors() throws RepositoryException {
    Value[] refs = getProperty("jcr:successors").getValues();
    Version[] successors = new Version[refs.length];
    for (int i = 0; i < refs.length; i++) {
      successors[i] = (Version) session.getNodeByUUID(refs[i].getString());
      //successors[i] = versionStorageManager().getVersionByUUID(refs[i].getString(), session);
    }
    return successors;
  }

  public Version[] getPredecessors() throws RepositoryException {
    Value[] refs = new Value[0];
    try {
      refs = getProperty("jcr:predecessors").getValues();
    } catch (PathNotFoundException e) {
    }
    Version[] ps = new Version[refs.length];
    for (int i = 0; i < refs.length; i++) {
      //ps[i] = versionStorageManager().getVersionByUUID(refs[i].getString(), session);
      ps[i] = (Version) session.getNodeByUUID(refs[i].getString());
    }
    return ps;
  }

  public void addSuccessor(Node successor) throws RepositoryException {
    Value[] refs = new Value[0];
    boolean firstSucessor = false;
    try {
      refs = getProperty("jcr:successors").getValues();
    } catch (PathNotFoundException e) {
      firstSucessor = true;
    }
    Value[] successors = new Value[refs.length + 1];
    for (int i = 0; i < refs.length; i++)
      successors[i] = refs[i];
    successors[refs.length] = session.getValueFactory().createValue(successor);
    PropertyImpl prop = createChildProperty("jcr:successors", successors,
        PropertyType.REFERENCE);
    if(firstSucessor)
      workspaceDataManager().add(prop);
    else
      workspaceDataManager().update(prop);
  }
  
  public void addPredecessor(Node predecessor) throws RepositoryException {
    Value[] refs = new Value[0];
    boolean firstPredecessor = false;
    try {
      refs = getProperty("jcr:predecessors").getValues();
    } catch (PathNotFoundException e) {
      firstPredecessor = true;
    }
    Value[] predecessors = new Value[refs.length + 1];
    for (int i = 0; i < refs.length; i++)
      predecessors[i] = refs[i];
    predecessors[refs.length] = session.getValueFactory().createValue(predecessor);
    PropertyImpl prop = createChildProperty("jcr:predecessors", predecessors,
        PropertyType.REFERENCE);
    if(firstPredecessor)
      workspaceDataManager().add(prop);
    else
      workspaceDataManager().update(prop);
  }
  
  public NodeImpl restore(NodeImpl node, boolean removeExisting) throws RepositoryException {
    
    NodeImpl frozenNode = (NodeImpl)getNode("jcr:frozenNode");
/*    
    String name = node.getName();
    String ptName = frozenNode.getProperty("jcr:frozenPrimaryType").getString();
    
    String[] mtNames = new String[0];
    Value[] mtValues = null;
    if(frozenNode.hasProperty("jcr:frozenMixinTypes")) {
      mtValues = frozenNode.getProperty("jcr:frozenMixinTypes").getValues();
      mtNames = new String[mtValues.length];
      for(int i=0; i<mtNames.length; i++)
        mtNames[i] = mtValues[i].getString();
    }
    
        // should have uuid
    String uuid = frozenNode.getProperty("jcr:frozenUuid").getString();

*/    
    // make new node from frozen
    NodeImpl parent = (NodeImpl)node.getParent();
    
    InternalQName name = node.getActualItemData().getQName();
    
    String ptNameStr = frozenNode.getProperty("jcr:frozenPrimaryType").getString();
    //NameValue ss = (NameValue)frozenNode.getProperty("jcr:frozenPrimaryType").getValue();
    
    //InternalQName ptName = InternalQName.parse(frozenNode.getProperty("jcr:frozenPrimaryType").getString());
    InternalQName ptName = ((NameValue)frozenNode.getProperty("jcr:frozenPrimaryType").getValue()).getQName();
    InternalQName[] mtNames = null;
    Value[] mtValues = null;
    if(frozenNode.hasProperty("jcr:frozenMixinTypes")) {
      mtValues = frozenNode.getProperty("jcr:frozenMixinTypes").getValues();
      mtNames = new InternalQName[mtValues.length];
      for(int i=0; i<mtNames.length; i++)
        mtNames[i] = ((NameValue)mtValues[i]).getQName();
        //mtNames[i] = InternalQName.parse(mtValues[i].getString());
    }
    String uuid = null;
    if(frozenNode.hasProperty("jcr:frozenUuid")) {
      uuid = frozenNode.getProperty("jcr:frozenUuid").getString();
    }


    
    
    // index should be correct remove here so
    node.remove();
    //NodeImpl restoredNode = parent.createChildNode(name, ptName, mtNames, uuid, true, false);

    NodeImpl restoredNode = parent.createChildNodeInmemory(name, ptName, mtNames, 
        uuid, -1, false, false, false, false);
    
    // visitor accept frozenNode
    // traverse and save state in WorkspaceDataManager
    RestoreVisitor visitor = new RestoreVisitor(restoredNode, removeExisting);
    frozenNode.accept(visitor);

    restoredNode.createChildProperty("jcr:primaryType", 
        new Value[] {session.getValueFactory().createValue(ptNameStr, PropertyType.NAME)}, PropertyType.NAME);
    restoredNode.createChildProperty("jcr:uuid", 
        new Value[] {session.getValueFactory().createValue(uuid)}, PropertyType.REFERENCE);
    if(mtValues != null)
      restoredNode.createChildProperty("jcr:mixinTypes", mtValues, PropertyType.NAME);

    restoredNode.createChildProperty("jcr:baseVersion", 
        new Value[] {session.getValueFactory().createValue(this)}, PropertyType.REFERENCE);
    restoredNode.createChildProperty("jcr:isCheckedOut",
        new Value[] {session.getValueFactory().createValue(false)}, PropertyType.BOOLEAN);


    NodeDumpVisitor nd = new NodeDumpVisitor();
    restoredNode.accept(nd);
    
    BatcheableWorkspaceDataManager dm = session.getTransientNodesManager().getWorkspaceDataManager(); 
    node.save();
    
  
    dm.beginSessionBatch((SessionImpl)node.getSession());
    //dm.delete(node);
    dm.add(restoredNode);
    dm.endSessionBatch((SessionImpl)node.getSession());
    
    
    // and return new node
    return restoredNode;
  }
  
  
  public boolean isSuccessorOrSameOf(VersionImpl anotherVersion) 
  throws RepositoryException {
    Version[] prds = getPredecessors();
    for(int i=0; i<prds.length; i++) {
      if(prds[i].getUUID().equals(anotherVersion.getUUID()) ||
         ((VersionImpl)prds[i]).isSuccessorOrSameOf(anotherVersion))
        return true;
    }
    return false;
  }
 
}