/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.SAXException;
import javax.jcr.Session;
import java.util.Stack;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.digester.Rule;
import org.exoplatform.services.jcr.impl.importing.CommonRule;
import javax.jcr.RepositoryException;

/**
 * Created by The eXo Platform SARL
 *  Common class for Digester Rules
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: CommonSysViewRule.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class CommonSysViewRule extends CommonRule
{


      private Stack nodesStack;
      private Session session;

      protected CommonSysViewRule(Stack nodesStack) {
          if (nodesStack == null)
              throw new NullPointerException("nodesStack can't be null");
          this.nodesStack = nodesStack;


          try{
              session = peekNode().getSession();
          }
          catch(RepositoryException ex){
              log.error("Can't get session from start node" );
              throw new RuntimeException("Can't get session from start node",ex);
          }



      }
      protected Stack getNodesStack(){
          return nodesStack;
      }

      protected Session getSession(){
          return session;
      }



    /**
     * Return top-most Element in digester stack
     * If top-most Element not XMLElementDefinition - return null
     */
    protected XMLElementDefinition peekElementDef() {
        XMLElementDefinition elementDef = null;
        Object peekObject = digester.peek();
        if (peekObject.getClass().isAssignableFrom(XMLElementDefinition.class)) {
            elementDef = (XMLElementDefinition) peekObject;
        }

        return elementDef;
    }



    /**
     * Check topmost element in stack for conforming next rules:
     *  -type element is XMLElementDefinition (or descendant)
     */
    protected void checkCurrentElementDefinition() throws SAXException{
        XMLElementDefinition elementDef = peekElementDef();
        if (elementDef == null){
            String errorText = "No Parent Element Definition, ["+
                               digester.peek().getClass().getName() +
                               "], instance ["+digester.peek().toString()+"]  otherwise";
            log.error(errorText);
            throw new SAXException(errorText);
        }
    }


    /**
     * Check topmost element in stack for conforming next rules:
     *  -type element is XMLElementDefinition (or descendant)
     *  -targetClass is getTargetClass()
     */
    protected void checkCurrentElementDefinition(Class targetClass) throws SAXException{
            checkCurrentElementDefinition();
            XMLElementDefinition elementDef = peekElementDef();
            if (!elementDef.isProperClass(targetClass)){
                String errorText = "Wrong Target class for Parent Element Definition."+
                                   "Wait for ["+targetClass.getName()+"] but found ["+
                                   elementDef.getTargetClass().getName()+"]. "+
                                   "Parent Instance dump ["+elementDef.toString()+"]";
                log.error(errorText);
                throw new SAXException(errorText);
            }
    }

    /**
     * Check topmost element in stack for conforming next rules:
     *  -type element is XMLElementDefinition (or descendant)
     *  -getNameespaceURI() equals to namespaceURI
     *  -getLocalName() equals localName
     */
    protected void checkCurrentElementDefinition(String namespaceURI,String localName) throws SAXException{
            checkCurrentElementDefinition();
            XMLElementDefinition elementDef = peekElementDef();
            if (!elementDef.getNamespaceURI().equals(namespaceURI)){
                String errorText = "Wrong Namespace URI for Parent Element Definition."+
                                   "Wait for ["+namespaceURI+"] but found ["+
                                   elementDef.getNamespaceURI()+"]. "+
                                   "Parent Instance dump ["+elementDef.toString()+"]";
                log.error(errorText);
                throw new SAXException(errorText);
            }


            if (!elementDef.getLocalName().equals(localName)){
                String errorText = "Wrong Local Name for Parent Element Definition."+
                                   "Wait for ["+localName+"] but found ["+
                                   elementDef.getLocalName()+"]. "+
                                   "Parent Instance dump ["+elementDef.toString()+"]";
                log.error(errorText);
                throw new SAXException(errorText);
            }
    }



    /**
     * Check topmost element in stack for conforming next rules:
     *  -type element is XMLElementDefinition (or descendant)
     *  -getNameespaceURI() equals to namespaceURI
     *  -getLocalName() equals localName
     *  -targetClass is getTargetClass()
     */
    protected void checkCurrentElementDefinition(String namespaceURI,String localName,Class targetClass) throws SAXException{
        checkCurrentElementDefinition(targetClass);
        checkCurrentElementDefinition(namespaceURI,localName);
    }



    protected XMLElementDefinition newElementDefinition(String namespaceURI,
            String localName,Attributes attributes, Class targetClass){
        return new XMLElementDefinition(namespaceURI,localName,
                    new AttributesImpl(attributes), targetClass);

    }


    /**
     * @todo check if exists mandatored attributes sv:name
     */
    protected void checkMandatoryAttribs(Attributes attributes) throws SAXException{

    }


    public void begin(String namespace, String name, Attributes attributes)
          throws Exception {
      super.begin(namespace, name, attributes);
      checkMandatoryAttribs(attributes);
  }


}
