/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.NamespaceException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;

import org.apache.commons.digester.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

//import org.exoplatform.services.jcr.impl.xml.XMLLogHandler;

/**
 * Created by The eXo Platform SARL
 *  Content handler for import.
 *  Recognize type of mapping xml (Sys or Doc view JSR-170 7.3)
 *   and create appopriate Digester
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: SessionImportContentHandler.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class SessionImportContentHandler implements ContentHandler//, DTDHandler, EntityResolver, ErrorHandler
{
  private static final Log log = LogFactory
      .getLog(SessionImportContentHandler.class);

  private SessionImpl session;

  private NodeImpl startNode;

  Map namespaceMap = new Hashtable();

  Digester digester = null;

  private int uuidBehavior = ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW;
  
  private NamespaceRegistry nsReg;


  public SessionImportContentHandler(SessionImpl session, NodeImpl startNode) 
  throws RepositoryException {
    log.debug("Create");
    this.session = session;
    this.startNode = startNode;
    this.nsReg = session.getWorkspace().getNamespaceRegistry();
  }

  /**
   * @see   javax.jcr.ImportUUIDBehavior
   */
  public int getUuidBehavior() {
    return uuidBehavior;
  }

  /**
   * @see   javax.jcr.ImportUUIDBehavior
   */
  public void setUuidBehavior(int uuidBehavior) {
    this.uuidBehavior = uuidBehavior;
  }


  //    private org.exoplatform.services.jcr.impl.xml.XMLLogHandler tempHandler =
  //            new org.exoplatform.services.jcr.impl.xml.XMLLogHandler();

  //    private ContentHandler getDigester() throws SAXException {
  //       return tempHandler;
  //    }

  private Digester getDigester() throws SAXException {
    if (digester == null)
      throw new SAXException("No digester choosed");
    return digester;
  }

  private boolean isInitDigester() {
    return digester != null;
  }

  private void initDigester(String namespaceURI, String qName)
      throws SAXException {
    if (!isInitDigester()) {
      log.debug("Choose appopriate method of import.");
      log.debug("First element of xml: namespaceURI[" + namespaceURI
          + "] qName[" + qName + "]");
      digester = new Digester();

      if ("http://www.jcp.org/jcr/sv/1.0".equals(namespaceURI)
          && qName.equals("sv:node")) {
        log.info("Input XML recognized as SYSTEM VIEW, uuidBehaviour "
            + getUuidBehavior());
        SysViewSessionRules rules = new SysViewSessionRules(startNode,
            getUuidBehavior());
        //                rules.setUuidBehavior(getUuidBehavior());
        digester.setRules(rules);
      } else {
        log.info("Input XML recognized as DOCUMENT VIEW");
        DocViewSessionRules rules = new DocViewSessionRules(startNode,
            getUuidBehavior());
        //                rules.setUuidBehavior(getUuidBehavior());
        digester.setRules(rules);
      }

      catchUpWithOccurredEvents();

    }
  }

  /**
   * Send evens which occured before
   *  digester created
   */
  private void catchUpWithOccurredEvents() throws SAXException {

    getDigester().startDocument();
    //namespaces
    Iterator namespaceIterator = namespaceMap.keySet().iterator();
    String prefixList = "";
    while (namespaceIterator.hasNext()) {
      String prefix = (String) namespaceIterator.next();
      getDigester().startPrefixMapping(prefix,
          (String) namespaceMap.get(prefix));
      prefixList += prefix + ";";
    }
    log
        .trace("catchUpWithOccurredEvents(): startDocument(); startPrefixMapping() - "
            + prefixList);

  }

  //-----------org.xml.sax.ContentHandler-----------------------------------------
  public void setDocumentLocator(Locator locator) {
    try {
      getDigester().setDocumentLocator(locator);
    } catch (SAXException ex) {
      new RuntimeException(ex.getMessage());
    }
  }

  public void startDocument() throws SAXException {
    log.trace("startDocument");
    //        getDigester().startDocument();
  }

  public void endDocument() throws SAXException {
    getDigester().endDocument();
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
    log.trace("startPrefixMapping prefix[" + prefix + "] uri[" + uri + "]");
    try {
      nsReg.getPrefix(uri);
    } catch (NamespaceException e) {
      try {
        nsReg.registerNamespace(prefix, uri);
      } catch (NamespaceException e1) {
        throw new RuntimeException(e1);
      } catch (RepositoryException e1) {
        throw new RuntimeException(e1);
      }
      
    } catch (RepositoryException e) {
      throw new RuntimeException(e);
    }

    if (isInitDigester()) {
      getDigester().startPrefixMapping(prefix, uri);
    } else {
      namespaceMap.put(prefix, uri);
    }
/*
    try {
      session.setNamespacePrefix(prefix, uri);
    } catch (RepositoryException ex) {
      log.error("Error on set namespace mapping to session");
      new SAXException("Error on set namespace mapping to session", ex);
    }
*/    
  }

  public void endPrefixMapping(String prefix) throws SAXException {
    getDigester().endPrefixMapping(prefix);
  }

  public void startElement(String namespaceURI, String localName, String qName,
      Attributes atts) throws SAXException {

    log.trace("startElement namespaceURI[" + namespaceURI + "] localName["
        + localName + "] qName[" + qName + "]");

    initDigester(namespaceURI, qName);
    getDigester().startElement(namespaceURI, localName, qName, atts);

  }

  public void endElement(String namespaceURI, String localName, String qName)
      throws SAXException {

    getDigester().endElement(namespaceURI, localName, qName);
  }

  public void characters(char ch[], int start, int length) throws SAXException {
    getDigester().characters(ch, start, length);
  }

  public void ignorableWhitespace(char ch[], int start, int length)
      throws SAXException {
    getDigester().ignorableWhitespace(ch, start, length);
  }

  public void processingInstruction(String target, String data)
      throws SAXException {
    getDigester().processingInstruction(target, data);
  }

  public void skippedEntity(String name) throws SAXException {
    getDigester().skippedEntity(name);
  }

  //-----------org.xml.sax.DTDHandler---------------------------------------------

  public void notationDecl(String name, String publicId, String systemId)
      throws SAXException {
    getDigester().notationDecl(name, publicId, systemId);
  }

  public void unparsedEntityDecl(String name, String publicId, String systemId,
      String notationName) throws SAXException {
    getDigester().unparsedEntityDecl(name, publicId, systemId, notationName);
  }

  //-----------org.xml.sax.EntityResolver-----------------------------------------

  public InputSource resolveEntity(String publicId, String systemId)
      throws SAXException, IOException {
    return getDigester().resolveEntity(publicId, systemId);
  }

  //-----------org.xml.sax.ErrorHandler-------------------------------------------

  public void warning(SAXParseException exception) throws SAXException {
    getDigester().warning(exception);
  }

  public void error(SAXParseException exception) throws SAXException {
    getDigester().error(exception);
  }

  public void fatalError(SAXParseException exception) throws SAXException {
    getDigester().fatalError(exception);
  }

}