/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import java.util.Iterator;
import java.util.Stack;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.Value;

import org.apache.commons.codec.binary.Base64;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Created by The eXo Platform SARL
 *  Used as rule vor sv:property element in org.apache.commons.digester
 *  on import repository content from System view (JSR-170 7.3.1)
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: SysViewPropertyRule.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class SysViewPropertyRule extends CommonSysViewRule {


   SysViewPropertyRule(Stack nodesStack){
       super(nodesStack);
   }

    public void begin(String namespace, String name, Attributes attributes) throws
            Exception {
        super.begin(namespace, name, attributes);

        checkIfExistsParentNode();

        XMLElementDefinition elementDef = newElementDefinition(
                namespace, name, attributes, javax.jcr.Property.class);
        peekElementDef().addChild(elementDef);
        digester.push(elementDef);
    }

    /**
     * @todo check if exists mandatored attributes sv:name and sv:type
     */

    private void checkIfExistsParentNode() throws SAXException{
        //check if exists parent Node
        try {
            checkCurrentElementDefinition(javax.jcr.Node.class);
        } catch (SAXException ex) {
            XMLElementDefinition parentElementDef = peekElementDef();
            String errorText = parentElementDef == null ? "" :
                               " (Parent [" + parentElementDef.toString() +
                               "])";
            errorText = "Document not valid, no found parent node." + errorText;
            log.error(errorText);
            throw new SAXException(errorText);

        }
    }


    public void body(String namespace, String name, String text) throws
            Exception {
        super.body(namespace, name, text);
        checkCurrentElementDefinition(namespace, name, javax.jcr.Property.class);
        if (!"".equals(text)){
            String errorText = "sv:property can't contain text body [" + text +
                               "]. Property dump [" + peekElementDef().toString() +
                               "]";

            log.error(errorText);
            throw new SAXException(errorText);
        }
    }


    public void end(String namespace, String name) throws Exception {
        super.end(namespace, name);
        checkCurrentElementDefinition(namespace, name,
                                      javax.jcr.Property.class);

        XMLElementDefinition propertyDef =
            (XMLElementDefinition)digester.pop();

        checkCurrentElementDefinition(Node.class);
        createNodeProperties(propertyDef);
    }

    private String getPropertyValue(XMLElementDefinition propertyDef) throws SAXException{
        try{
            Iterator valueIterator = propertyDef.getChildrenIterator();

            XMLElementDefinition valueDef =
                    (XMLElementDefinition)valueIterator.next();

            if (valueIterator.hasNext()){
                throw new SAXException("Primary type property have more than one value");
            }

            return valueDef.getBody();
        }
        catch(java.util.NoSuchElementException ex){
            log.error("No value for primary type property ",ex);
            throw new SAXException("No value for primary type property ");
        }
        catch(java.lang.ClassCastException ex){
            log.error("Wrong type of property value definition",ex);
            throw new SAXException("Wrong type of property value definition ["+
                    propertyDef.getChildrenIterator().next().getClass().getName()+"]");
        }

    }

    /**
     * topmost definition is node
     */
    private Node createNode(XMLElementDefinition primaryTypePropertyDef) throws
            javax.jcr.ItemExistsException,
            javax.jcr.PathNotFoundException,
            javax.jcr.nodetype.NoSuchNodeTypeException,
            javax.jcr.lock.LockException,
            javax.jcr.version.VersionException,
            javax.jcr.nodetype.ConstraintViolationException,
            javax.jcr.RepositoryException,
            SAXException{

        String propertyType =
            primaryTypePropertyDef.getAttributes().getValue("sv:type");
        String primaryNodeTypeName = getPropertyValue(primaryTypePropertyDef);

        if (!PropertyType.TYPENAME_NAME.equals(propertyType))
            throw new SAXException("Property type of jcr:primaryType expected name [Name], but found ["+propertyType+"]");

        XMLElementDefinition nodeDefinition = peekElementDef();

        String newNodeName = nodeDefinition.getAttributes().getValue("sv:name");

        Node newNode = createNode(newNodeName,primaryNodeTypeName);
        nodeDefinition.setProceed(true);

//        log.debug("create node ["++"] with primaryNodeTypeName["+
//                primaryNodeTypeName+"]");

//        Node currentNode = (Node) nodesStack.peek();

//        Node newNode = currentNode.addNode(newNodeName,primaryNodeTypeName);

//        nodesStack.push(newNode);
        return newNode;

    }



    /**
     * Set node mixin types
     */
    private void setNodeMixinTypes(XMLElementDefinition propertyDef) throws
            SAXException,javax.jcr.nodetype.NoSuchNodeTypeException,
   javax.jcr.ValueFormatException, javax.jcr.RepositoryException {

    checkNodeCreated();
        Node node = peekNode();

//        Node node = (Node) nodesStack.peek();
        Value[] valueArray = getValueArray(propertyDef);
        for (int i=0; i<valueArray.length; i++){
            log.trace("set Mixin type["+valueArray[i].getString()+"]");
            node.addMixin(valueArray[i].getString());
        }
    }

    /**
     * Read child XMLElementDefinition-s of property,
     *   and return array of javax.jcr.Value
     */
    private Value[] getValueArray(XMLElementDefinition
                                                  propertyDef) throws
                            javax.jcr.ValueFormatException,
                            javax.jcr.RepositoryException,
                            SAXException{
        String propertyTypeName = propertyDef.getAttributes().getValue("sv:type");
        Value[] valueArray = new Value[propertyDef.getChildrenCount()];
        Iterator valueDefIterator = propertyDef.getChildrenIterator();
                int i = 0;
                while (valueDefIterator.hasNext()) {
                    XMLElementDefinition valueDef =
                            (XMLElementDefinition) valueDefIterator.next();

                    if (! valueDef.isProperClass(Value.class) )
                        throw new SAXException("Wrong child of property ("+propertyDef+")");

                    //if binary then use base64
                    if (PropertyType.TYPENAME_BINARY.equals(propertyTypeName)){
                        valueArray[i] = new org.exoplatform.services.jcr.impl.core.value.BinaryValue(
                            Base64.decodeBase64(valueDef.getBody().getBytes()));

                    }
                    else{
                        valueArray[i] = this.getSession().getValueFactory().
						createValue(valueDef.getBody(), ExtendedPropertyType.valueFromName(propertyTypeName)); 
//                        	PropertyConvertor.convert(
//                                new org.exoplatform.services.jcr.impl.core.value.StringValue(valueDef.getBody()),
//                                 PropertyType.valueFromName(propertyTypeName));
                    }
                    i++;
                }
        return valueArray;
    }




    /**
     * @todo binary value encoding
     * @todo property cast
     */
    private Property createPropertyFromDefinition(XMLElementDefinition
                                                  propertyDef) throws javax.jcr.
            ValueFormatException,
            javax.jcr.version.VersionException,
            javax.jcr.lock.LockException,
            javax.jcr.nodetype.ConstraintViolationException,
            javax.jcr.RepositoryException,
            SAXException {

        checkNodeCreated();

        String propertyName = propertyDef.getAttributes().getValue("sv:name");
        String propertyTypeName = propertyDef.getAttributes().getValue("sv:type");


        Value[] valueArray = getValueArray(propertyDef);
        if (valueArray.length == 1) {
            return createProperty(propertyName,
                                  valueArray[0],
                                  ExtendedPropertyType.valueFromName(propertyTypeName));
        } else {
            return createProperty(propertyName,
                                  valueArray,
                                  ExtendedPropertyType.valueFromName(propertyTypeName));
        }
    }


    private void checkNodeCreated() throws SAXException {
        checkCurrentElementDefinition(Node.class);

        if (!peekElementDef().isProceed()) {
            String errorText = "Not found jcr:primaryType for node, must " +
                               "be first elem (" + peekElementDef().toString() +
                               ")";
            log.error(errorText);
            throw new SAXException(errorText);
        }
    }

    /**
     * topmost definition is node
     */
    private void createNodeProperties(XMLElementDefinition propertyDef) throws
            Exception {

        String propertyName = propertyDef.getAttributes().getValue("sv:name");
        String propertyType = propertyDef.getAttributes().getValue("sv:type");
        log.debug("Init property. name[" + propertyName + "] type[" +
                  propertyType + "] " +
                  attsSnapshot(propertyDef.getAttributes()));

        if ("jcr:primaryType".equals(propertyName) )
            createNode(propertyDef);
        else
//        if ("jcr:uuid".equals(propertyName)) {}
//        else
//        if ("jcr:created".equals(propertyName)) {}
//        else
        if ("jcr:mixinTypes".equals(propertyName))
            setNodeMixinTypes(propertyDef);
        else
            createPropertyFromDefinition(propertyDef);
    }


}
