/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.jndi;

import org.apache.commons.collections.map.ReferenceMap;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.RefAddr;
import javax.naming.spi.ObjectFactory;
import java.util.Hashtable;
import java.util.Map;

import org.exoplatform.services.jcr.impl.StandaloneContainer;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: BindableRepositoryFactory.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class BindableRepositoryFactory implements ObjectFactory {

  private static Map cache = new ReferenceMap(ReferenceMap.HARD,
      ReferenceMap.SOFT);

  public BindableRepositoryFactory() {
  }

  public Object getObjectInstance(Object obj, Name name, Context nameCtx,
      Hashtable environment) throws Exception {
    if (obj instanceof Reference) {
      Reference ref = (Reference) obj;
      synchronized (cache) {
        if (cache.containsKey(ref)) {
          return cache.get(ref);
        } else {
          RefAddr contCfg = ref
              .get(BindableRepositoryImpl.CONTAINERCONFIG_ADDRTYPE);
          String configName = (String) ref.get(
              BindableRepositoryImpl.REPOSITORYCONFIG_ADDRTYPE).getContent();
          if (contCfg != null) {
            // here the code will work properly only when no StandaloneContainer instance created yet
            org.exoplatform.services.jcr.impl.StandaloneContainer
                .setConfigurationURL((String) contCfg.getContent());
          }
          BindableRepositoryImpl rep = new BindableRepositoryImpl(
              StandaloneContainer.getInstance(), configName);
          cache.put(ref, rep);
          return rep;
        }
      }
    }
    return null;
  }
}
