/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.core.query;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.Properties;

import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.SimpleCredentials;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.Workspace;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.UUIDGenerator;
import org.exoplatform.services.jcr.impl.core.query.lucene.FieldNames;
import org.exoplatform.services.jcr.impl.core.query.lucene.LuceneQueryBuilder;
import org.exoplatform.services.jcr.impl.core.query.lucene.SearchIndex;
import org.exoplatform.services.jcr.impl.core.query.xpath.XPathQueryBuilder;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;
import org.exoplatform.services.jcr.impl.core.value.StringValue;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: IndexingTest.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class IndexingTest extends TestCase {

  protected String getRepositoryName() {
    return REPOSITORY_NAME;
  }

  private static final String REPOSITORY_NAME = "db1";//"mock";

  private static final String WORKSPACE_NAME = "ws";

  protected Log log = LogFactory.getLog(this.getClass());

  protected static final String STRING_VALUE = " In this talk, Rod will draw on his experience as a consultant to discuss why J2EE projects "
      + "often fail, and how to avoid common pitfalls. The main focus will be technical issues, such "
      + "as architectural choices and coding practices. Rod will also discuss organization issues such "
      + "as approaches to team management and tool set, and process issues such as appropriate risk "
      + "mitigation and testing strategy. The presentation will conclude with positive recommendations "
      + "that may help to make your projects more successful.";

  protected static final String BINARY_STRING_VALUE = " The eXo platform  code source is available for all our license types (GNU/GPL or commercial). "
      + "Indeed, we firmly believe that Open Source software presents undeniable advantages for the client, "
      + "and thereby give our company an important competitive edge in the portal industry: Firstly, Open Source"
      + "software (OSS) offers a very high standard of quality as the code is reviewed by the open source community "
      + "that has adopted and drives most of the best practises of the industry, such as XP programming and the use "
      + "of design patterns. j2ee";

  protected static final byte[] BINARY_BYTE_VALUE = new byte[] { 0, 1, 2, 3, 4,
      5, 6, 7, 8, 9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0x20, 0x4A, 0x32, 0x45, 0x45 };

  //                     ' '   'J'   '2'   'E'   'E'
  protected static final Calendar DATE_VALUE = new GregorianCalendar(2005, 06,
      27, 15, 10); // 2005-07-27T15:10

  protected static final long LONG_VALUE = 1234567890l;

  protected static final double DOUBLE_VALUE = 789.15d;

  protected static final boolean BOOLEAN_VALUE = false;

  protected static final String NAME_VALUE = "node_1_1";

  protected static final String PATH_VALUE = "/root-node/node_1/node_1_1";

  protected Value stringValue;

  protected Value binaryValue;

  protected Value binaryStringValue;

  protected Value dateValue;

  protected Value longValue;

  protected Value doubleValue;

  protected Value booleanValue;

  protected Value nameValue;

  protected Value pathValue;
  
  protected SessionImpl session;
  protected RepositoryImpl repository;
  protected SimpleCredentials credentials;
//  protected ExoContainer servicesManager;
  protected PortalContainer servicesManager;
  protected Workspace workspace;
  protected RepositoryService repositoryService;
  protected LogService logService;
  protected Node root;
  protected ValueFactory valueFactory;
  
  
  private static final String TEST_IND_DIR = "target/test/index";


  public void setUp() throws Exception {

  	logService = (LogService) RootContainer.getInstance().
          getComponentInstanceOfType(LogService.class);

      log = logService.getLog("org.exoplatform.services.jcr"); 

      logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG, true);

      servicesManager = RootContainer.getInstance().getPortalContainer("exo");
      
      if(System.getProperty("java.security.auth.login.config") == null)
          System.setProperty("java.security.auth.login.config", "src/resources/login.conf" );

      credentials = new SimpleCredentials("exo", "exo".toCharArray());

      repositoryService =
        (RepositoryService) servicesManager.getComponentInstanceOfType(RepositoryService.class);

      repository = (RepositoryImpl) repositoryService.getRepository(getRepositoryName());

      //repository.getContainer().start();
      
      if(!((RepositoryImpl) repository).isWorkspaceInitialized("ws"));
        ((RepositoryImpl) repository).initWorkspace("ws", "nt:unstructured"); 
        
      if(!((RepositoryImpl) repository).isWorkspaceInitialized("ws2"));
        ((RepositoryImpl) repository).initWorkspace("ws2", "nt:unstructured");

      session = (SessionImpl)repository.login(credentials, "ws");
      workspace = session.getWorkspace();
      root = session.getRootNode();
      valueFactory = session.getValueFactory();

      logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG, true);

      initRepository();

  }

  //    protected String binaryStringValue;

  public void initRepository() throws RepositoryException {

    logService = (LogService) RootContainer.getInstance()
        .getComponentInstanceOfType(LogService.class);

    Node workspaceRoot = session.getRootNode();

    Node root = workspaceRoot.addNode("root-node", "nt:unstructured");

    Node node_1 = root.addNode("node_1", "nt:unstructured");
    Node node_2 = root.addNode("node_2", "nt:unstructured");
    Node node_3 = root.addNode("node_3", "nt:folder");

    Node node_2_1 = node_2.addNode("node_2_1", "nt:unstructured");
    Node node_2_2a = node_2.addNode("node_2_2", "nt:unstructured");
    Node node_2_2b = node_2.addNode("node_2_2", "nt:unstructured");
    Node node_2_2b_1 = node_2_2b.addNode("node_2_2b_1", "nt:unstructured");
    Node node_2_2c = node_2.addNode("node_2_2", "nt:unstructured");
    Node node_2_3 = node_2.addNode("node_2_3", "nt:unstructured");

    Node node_1_1 = node_1.addNode("node_1_1", "nt:unstructured");
    Node node_1_2 = node_1.addNode("node_1_2", "nt:unstructured");
    Node node_1_3 = node_1.addNode("node_1_3", "nt:unstructured");
    Node node_1_4 = node_1.addNode("node_1_4", "nt:unstructured");
    Node node_1_5 = node_1.addNode("node_1_5", "nt:folder");

    Node node_1_3_1 = node_1_3.addNode("node_1_3_1", "nt:unstructured");
    Node node_1_3_2 = node_1_3.addNode("node_1_3_2", "nt:unstructured");
    Node node_1_3_3 = node_1_3.addNode("node_1_3_3", "nt:unstructured");

    //create Values
    java.util.Calendar calendar = java.util.Calendar.getInstance();
    calendar.setTimeInMillis(System.currentTimeMillis());

    stringValue = new StringValue(STRING_VALUE);
    //          binaryStringValue = "binaryValue "+System.currentTimeMillis();
    binaryStringValue = new BinaryValue(BINARY_STRING_VALUE);
    binaryValue = new BinaryValue(BINARY_BYTE_VALUE);

    dateValue = valueFactory.createValue(DATE_VALUE);
    longValue = valueFactory.createValue(LONG_VALUE);
    doubleValue = valueFactory.createValue(DOUBLE_VALUE);
    booleanValue = valueFactory.createValue(BOOLEAN_VALUE);
    nameValue = valueFactory.createValue(NAME_VALUE, PropertyType.NAME);
    pathValue = valueFactory.createValue(PATH_VALUE, PropertyType.PATH);

    //add property
    node_1_1.setProperty("stringValue", stringValue);
    node_1_1.setProperty("dateValue", dateValue);
    Value val1 = valueFactory.createValue("test1");
    Value val2 = valueFactory.createValue("test2");
    node_1_1.setProperty("multiValue", new Value[] { val1, val2 });

    node_1_2.setProperty("stringValue", stringValue);
    node_1_2.setProperty("longValue", longValue);

    node_1_4.setProperty("doubleValue", doubleValue);
    node_1_4.setProperty("booleanValue", booleanValue);
    node_1_4.setProperty("nameValue", nameValue);
    assertNotNull("nameValue", node_1_4.getProperty("nameValue").getValue());
    node_1_4.setProperty("pathValue", pathValue);
    //      assertNotNull("pathValue before save",node_1_4.getProperty("pathValue").getValue());

    node_2.setProperty("binaryValue", binaryValue);
    node_2_3.setProperty("binaryValue", binaryStringValue);

    //      assertNotNull("pathValue after save",node_1_4.getProperty("pathValue").getValue());
    session.save();
  }
/*
  public void testLuceneNodeIndexer() throws Exception {
    NodeImpl root = (NodeImpl) session.getRootNode();
    IndexWriter writer = new IndexWriter(TEST_IND_DIR,
        new StandardAnalyzer(), true);

    LocationFactory factory = repository.getSystemSession()
        .getLocationFactory();
    Document doc = NodeIndexer.createDocument((NodeData) root
        .getActualItemData(), factory);
    writer.addDocument(doc);
    //        printDoc(doc);

    NodeImpl n = (NodeImpl) root.getNode("root-node");
    doc = NodeIndexer.createDocument((NodeData) n.getActualItemData(), factory);
    writer.addDocument(doc);
    
    n = (NodeImpl) root.getNode("root-node/node_1");
    doc = NodeIndexer.createDocument((NodeData) n.getActualItemData(), factory);
    writer.addDocument(doc);

    NodeImpl n1 = (NodeImpl) root.getNode("root-node/node_1/node_1_1");
    n1.getProperties();
    doc = NodeIndexer.createDocument((NodeData) n1.getActualItemData(), factory);
    writer.addDocument(doc);
    //printDoc(doc);

    writer.optimize();
    writer.close();

    //System.out.println("DOC >>>>>>> "+doc.getField(FieldNames.UUID).stringValue());

    Searcher searcher = new IndexSearcher(TEST_IND_DIR);
    Analyzer analyzer = new StandardAnalyzer();

    Query query = org.apache.lucene.queryParser.QueryParser.parse(n1.getActualItemData().getUUID(),
        FieldNames.UUID, analyzer);
    //    	System.out.println("Searching for: " + query.toString(n.getActualItemData().getUUID()));
    Hits hits = searcher.search(query);
    assertEquals(1, hits.length());
    //    	System.out.println(hits.length() + " total matching documents");

    query = org.apache.lucene.queryParser.QueryParser.parse("successful", FieldNames.FULLTEXT, analyzer);
    //    	System.out.println("Searching for: " + "successful");
    hits = searcher.search(query);
    assertEquals(1, hits.length());
    //    	System.out.println(hits.length() + " total matching documents");
  }
*/
  
  public void testCreateLuceneSeachIndex() throws Exception {
    
    Properties props = new Properties();
    props.setProperty("indexDir", TEST_IND_DIR);
    //props.setProperty("workspaceName", "ws");
    
    LocationFactory factory = repository.getSystemSession()
        .getLocationFactory();

    //we.setQueryHandler()
//    SearchIndex searchIndex = new SearchIndex(entry);
    SearchIndex searchIndex = new SearchIndex(props);

    QueryHandlerContext context = new QueryHandlerContext(session
        .getTransientNodesManager().getWorkspaceDataManager(),
        UUIDGenerator.ROOT_UUID, factory);
    searchIndex.init(context);

    ArrayList removed = new ArrayList();
    removed.add(UUIDGenerator.ROOT_UUID);
    ArrayList added = new ArrayList();
    NodeImpl root = (NodeImpl)session.getRootNode();
    added.add(root.getActualItemData());

    searchIndex.updateNodes(removed.iterator(), added.iterator());
    searchIndex.close();
    
    
    System.out.println("Index Path " + searchIndex.getPath());
    

  }

  public void testUseLuceneSeachIndex() throws Exception {
    Properties props = new Properties();
    props.setProperty("indexDir", TEST_IND_DIR);
    LocationFactory factory = repository.getSystemSession()
        .getLocationFactory();
    SearchIndex searchIndex = new SearchIndex(props);
    QueryHandlerContext context = new QueryHandlerContext(session
        .getTransientNodesManager().getWorkspaceDataManager(),
        UUIDGenerator.ROOT_UUID, factory);
    searchIndex.init(context);
  
 //////////////////////// Lucene  
    
    IndexReader reader = searchIndex.getIndexReader();
    Searcher searcher = new IndexSearcher(reader);
    
    System.out.println("Read DOCSSSSSSSSSSSS "+reader.numDocs());
/*
    TermDocs docs=reader.termDocs();

    while(docs.next()) {
      System.out.println("DOC "+docs.doc()+" "+docs.freq());
      System.out.println("DOC "+reader.document(docs.doc()));
    }
*/    
//    TermEnum terms=reader.terms();

//    while(terms.next()) {
//      System.out.println("TERM "+terms.term().field()+" "+terms.term().text());
//    }
    
    //System.out.println("ISO>>> "+ISO9075.encode(UUIDGenerator.ROOT_UUID));
    
    Analyzer analyzer = searchIndex.getAnalyzer();

    Query q = org.apache.lucene.queryParser.QueryParser.parse("nt",
        FieldNames.LABEL, analyzer);
    System.out.println("QUERY "+q);
    Hits hits = searcher.search(q);
    //assertEquals(1, hits.length());
    System.out.println(hits.length() + " total matching documents");
    
    q = org.apache.lucene.queryParser.QueryParser.parse(UUIDGenerator.ROOT_UUID,
        FieldNames.PARENT, analyzer);
    hits = searcher.search(q);
    System.out.println("QUERY "+q);
    System.out.println(hits.length() + " total matching documents");
    
    q = org.apache.lucene.queryParser.QueryParser.parse(UUIDGenerator.ROOT_UUID,
        FieldNames.UUID, analyzer);
    hits = searcher.search(q);
    System.out.println("QUERY "+q);
    System.out.println(hits.length() + " total matching documents");

    
    //////////////////////////
    
    QueryRootNode qq;
    qq = XPathQueryBuilder.createQuery("/root-node/*[@jcr:primaryType='nt:folder']", factory);
  System.out.println("Xpath Query >>>> "+qq.dump());
    
//    qq = new QueryRootNode();
//    System.out.println("Xpath Query >>>> "+qq.dump());
    
    
    /*
    PathQueryNode path = new PathQueryNode(qq);
    LocationStepQueryNode locStep = new 
    LocationStepQueryNode(path, InternalQName.parse("[]root-node"), false); 
    path.addPathStep(locStep);
    locStep = new 
    LocationStepQueryNode(path, InternalQName.parse("[]node_1"), false); 
    path.addPathStep(locStep);
    qq.setLocationNode(path);
    */
    
    Query lq = LuceneQueryBuilder.createQuery(qq, session, factory, analyzer);
    System.out.println("Lucene Query >>>> "+lq.toString());
    
    hits = searcher.search(lq);
    System.out.println(hits.length() + " total matching documents");
    for(int i=0; i<hits.length(); i++) {
      
      //Enumeration flds = hits.doc(i).fields();
      //while(flds.hasMoreElements()) {
      //  System.out.println(" fld: "+(Field)flds.nextElement());
      //}
      
      String uuid = hits.doc(i).get(FieldNames.UUID);
      System.out.println(" doc: "+session.getTransientNodesManager().getItemByUUID(uuid).getPath());
      
      
    }
    
/*
    QueryImpl query = (QueryImpl)searchIndex.createExecutableQuery(session, 
        "/nt", javax.jcr.query.Query.XPATH);
    System.out.println("Query >>>> "+query);

     QueryResult result = query.execute();
     System.out.println("Result >>>> "+result.getRows().getSize());
*/    
  }
  
  private void printDoc(Document doc) {
    Enumeration flds = doc.fields();
    while (flds.hasMoreElements()) {
      Field f = (Field) flds.nextElement();
      System.out.println("Fileld >>>> " + f.name() + " = " + f.stringValue());
    }
  }

  /*
   private void assertNodesFound(String message, String[] paths, HitPageList hitPageList) throws Exception {
   List sourceList = new ArrayList();

   for (int i = 0; i < paths.length; i++)
   sourceList.add(paths[i]);

   List destList = getPathList(hitPageList);

   Collections.sort(sourceList);
   Collections.sort(destList);
   log.trace("Compare nodes ["+message+"] source.size ["+sourceList.size()+"] dest.size["+destList.size()+"]");
   int i = 0;
   for (i = 0; i < sourceList.size() && i < destList.size(); i++) {
   log.trace("Compare nodes ["+message+"] #"+i+" source ["+sourceList.get(i)+"] dest["+destList.get(i)+"]");
   if (! sourceList.get(i).equals(destList.get(i))){
   if (destList.contains(sourceList.get(i)))
   fail(message + ". Founded extra paths [" + destList.get(i) + "]");
   else
   fail(message + ". Not find path ["+ sourceList.get(i)+"]");
   }
   }

   if (i < sourceList.size()) {
   String oddPaths = "";
   for (int j=i; j < sourceList.size(); j++) {
   oddPaths += sourceList.get(j) + ";";
   }
   fail(message + ".  Not find paths  [" + oddPaths + "]");
   }

   if (i < destList.size()) {
   String oddPaths = "";
   for (int j=i; j < destList.size(); j++) {
   oddPaths += destList.get(j) + ";";
   }
   fail(message + ". Founded extra paths [" + oddPaths + "]");
   }
   }

   private List getPathList(HitPageList hitPageList) throws Exception {
   List col = new ArrayList();

   List hitsList = hitPageList.getAll();
   for (Iterator hitsIterator = hitsList.iterator();hitsIterator.hasNext();){
   Document doc = (Document) hitsIterator.next();
   log.trace("doc ["+doc.toString()+"]");
   col.add(doc.getField(QueryIndexerPlugin.FIELD_PATH).stringValue());
   }
   return col;
   }



   public List getSearchInputByPropertyNameAndKeyword(String propertyName, String keyword) {
   SearchInput searchInputPropName =
   new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, propertyName + '\uFFFF'+keyword);
   //        SearchInput searchInputKeyword =
   //                new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, keyword);

   List list = new ArrayList();
   list.add(searchInputPropName);
   //        list.add(searchInputKeyword);

   return list;
   }

   public void testFindUsePropertyName() throws Exception {
   WorkspaceImpl workspaceImpl = (WorkspaceImpl) workspace;
   Searcher searcher = workspaceImpl.getIndexSearcher();
   assertNotNull("searcher", searcher);

   HitPageList hitPageList;
   List listSearchInput;

   listSearchInput = getSearchInputByPropertyNameAndKeyword("stringValue","experience");
   assertNotNull("listSearchInput",listSearchInput);
   //        assertEquals("listSearchInput.size()",2,listSearchInput.size());
   hitPageList = searcher.search(listSearchInput);
   assertNodesFound("Part of stringValue",
   new String[]{"/root-node/node_1/node_1_2", "/root-node/node_1/node_1_1"}, hitPageList);
   listSearchInput = getSearchInputByPropertyNameAndKeyword("notStringValue","experience");
   assertEquals("text", 0, hitPageList.getAll().size());
   }


   public void testNodeTypes() throws Exception {
   WorkspaceImpl workspaceImpl = (WorkspaceImpl) workspace;
   Searcher searcher = workspaceImpl.getIndexSearcher();
   assertNotNull("searcher", searcher);


   HitPageList hitPageList;


   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_NODE_TYPE, "nt:hierarchyNode"));
   assertNodesFound("nt:hierarchyNode",
   new String[]{"/root-node/node_1/node_1_5",
   "/root-node/node_3"}, hitPageList);
   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_NODE_TYPE, "nt:folder"));
   assertNodesFound("nt:folder",
   new String[]{"/root-node/node_1/node_1_5",
   "/root-node/node_3"}, hitPageList);

   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_NODE_TYPE, "nt:unstructured"));
   log.debug("Find nt:unstructured node "+hitPageList.getAll().size());
   assertTrue("nt:unstructured",hitPageList.getAll().size()>5);
   }


   public void testSearchInvalidContain() throws Exception {
   WorkspaceImpl workspaceImpl = (WorkspaceImpl) workspace;
   Searcher searcher = workspaceImpl.getIndexSearcher();
   assertNotNull("searcher", searcher);

   HitPageList hitPageList;

   hitPageList = searcher.search(new SingleFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "there is not such text"));
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());


   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "2005-07-27T18:12"));//wrong time
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());

   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "1275"));//wrong long
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());

   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "true"));//wrong boolean
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());

   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "node_2"));//wrong anme
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());

   hitPageList = searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "/not-root-node"));//wrong path
   assertNotNull("hitPageList", hitPageList);
   assertEquals("text", 0, hitPageList.getAll().size());


   }



   public void testSearchByPath() throws Exception {
   WorkspaceImpl workspaceImpl = (WorkspaceImpl) workspace;
   Searcher searcher = workspaceImpl.getIndexSearcher();
   assertNotNull("searcher", searcher);

   HitPageList hitPageList;

   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PATH, "/root-node"));
   assertNotNull("hitPageList", hitPageList);
   assertNodesFound("root-node", new String[]{"/root-node"}, hitPageList);


   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PATH, "/root-node/node_1"));
   assertNotNull("hitPageList", hitPageList);
   assertNodesFound("node_1e",
   new String[]{"/root-node/node_1"}, hitPageList);

   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PATH, "/root-node/node_1/node_1_1"));
   assertNotNull("hitPageList", hitPageList);
   assertNodesFound("node_1_1",
   new String[]{"/root-node/node_1/node_1_1"}, hitPageList);

   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PATH, "/root-node/node_2/node_2_2"));
   assertNotNull("hitPageList", hitPageList);
   assertNodesFound("node_2_2",
   new String[]{"/root-node/node_2/node_2_2"}, hitPageList);

   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PATH, "/root-node/node_2/node_2_2[2]"));
   assertNotNull("hitPageList", hitPageList);
   assertNodesFound("node_2_2[2]",
   new String[]{"/root-node/node_2/node_2_2[2]"}, hitPageList);


   }



   public void testSearchValues() throws Exception {
   WorkspaceImpl workspaceImpl = (WorkspaceImpl) workspace;
   Searcher searcher = workspaceImpl.getIndexSearcher();
   assertNotNull("searcher", searcher);


   HitPageList hitPageList;

   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "experience"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("Part of stringValue",
   new String[]{"/root-node/node_1/node_1_2", "/root-node/node_1/node_1_1"}, hitPageList);


   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "experience"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("Part of stringValue",
   new String[]{"/root-node/node_1/node_1_2", "/root-node/node_1/node_1_1"}, hitPageList);





   hitPageList =
   //                 searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "dateValue?2005-07-27T15:10:00.000+03:00"));
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY,"dateValue"+ '\uFFFF'+"2005-07-27T15:10:00.000+03:00"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("dateValue",
   new String[]{"/root-node/node_1/node_1_1"}, hitPageList);

   hitPageList =
   searcher.search(new WildcardInputSearch(QueryIndexerPlugin.FIELD_PROPERTY, "dateValue?2005-07-27*"));


   assertNodesFound("Part of dateValue",
   new String[]{"/root-node/node_1/node_1_1"}, hitPageList);



   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "longValue"+ '\uFFFF'+LONG_VALUE));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("longValue",
   new String[]{"/root-node/node_1/node_1_2"}, hitPageList);




   hitPageList =
   searcher.search(new WildcardInputSearch(QueryIndexerPlugin.FIELD_PROPERTY, "longValue"+ '\uFFFF'+"123*"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("part longValue",
   new String[]{"/root-node/node_1/node_1_2"}, hitPageList);


   hitPageList =
   searcher.search(new WildcardInputSearch(QueryIndexerPlugin.FIELD_PROPERTY, "doubleValue\uFFFF789.15*"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("doubleValue",
   new String[]{"/root-node/node_1/node_1_4"}, hitPageList);


   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "booleanValue\uFFFFfalse"));

   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("booleanValue",
   new String[]{"/root-node/node_1/node_1_4"}, hitPageList);


   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "nameValue"+ '\uFFFF'+"node_1_1"));



   assertNodesFound("nameValue",
   new String[]{"/root-node/node_1/node_1_4"}, hitPageList);


   hitPageList =
   searcher.search(new KeywordFieldSearchInput(QueryIndexerPlugin.FIELD_PROPERTY, "pathValue"+ '\uFFFF'+"/root-node/node_1/node_1_1"));
   assertNotNull("hitPageList", hitPageList);

   assertNodesFound("pathValue",
   new String[]{"/root-node/node_1/node_1_4"}, hitPageList);



   //         protected static final Calendar DATE_VALUE = new GregorianCalendar(2005, 06, 27, 15, 10);
   //             protected static final long LONG_VALUE = 1234567890l;
   //             protected static final double DOUBLE_VALUE = 789.15d;
   //             protected static final boolean BOOLEAN_VALUE = false;
   //             protected static final String NAME_VALUE = "node_1_1";
   //             protected static final String PATH_VALUE = "/root-node/node_1/node_1_1";
   }





   public void tearDown() throws Exception {

   Node root = session.getRootNode();
   root.getNode("root-node").remove();
   session.save();

   }

   */

}