/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.database.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.exoplatform.container.configuration.ConfigurationException;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.PropertiesParam;
import org.exoplatform.container.xml.Property;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.database.HibernateConfiguration;
import org.exoplatform.services.database.HibernateConfigurationsService;
import org.exoplatform.services.log.LogService;

import org.hibernate.cfg.Configuration;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: $
 * 
 * @see HibernateConfigurationsService
 * 
 * Manages set of hibernate configurations
 */

public class HibernateConfigurationsServiceImpl implements HibernateConfigurationsService {
  
  private Map configurations;
  private String defPortalConfig;
  
  public HibernateConfigurationsServiceImpl(InitParams initParams, 
                                            ExoCacheProvider cacheProvider, 
                                            LogService logService) throws ConfigurationException {
    configurations = new HashMap();
    Iterator i = initParams.getPropertiesParamIterator();
    while (i.hasNext()) {
      PropertiesParam param = (PropertiesParam) i.next();
      String name = param.getName();
      HibernateSettingsFactory settingsFactory =  new HibernateSettingsFactory(cacheProvider);
      Configuration conf = new HibernateConfigurationImpl(settingsFactory);
      Iterator properties = param.getPropertyIterator();
      while (properties.hasNext()) {
        Property p = (Property) properties.next();
        conf.setProperty(p.getName(), p.getValue());
      }
      configurations.put(name, conf);
    }
    ValueParam defPortalParam = initParams.getValueParam("default-portal-configuration");
    if(defPortalParam == null) {
      throw new ConfigurationException("No default portal configuration defined!");
    } 
    defPortalConfig = defPortalParam.getValue();
  }
  
  
  /** 
   * @see org.exoplatform.services.database.HibernateConfigurationsService#getHibernateConfiguration(java.lang.String)
   */
  public HibernateConfiguration getHibernateConfiguration(String name) throws ConfigurationException {
    if(!configurations.containsKey(name))
      throw new ConfigurationException("Hibernate configuration not found for "+name);
    return (HibernateConfiguration) configurations.get(name);
  }
  
  public HibernateConfiguration getDefaultPortalConfiguration() throws ConfigurationException {
    return getHibernateConfiguration(defPortalConfig);
  }
  
  public void closeAllSessions(){
    Collection values = configurations.values();
    for (Iterator iter = values.iterator(); iter.hasNext();) {
      HibernateConfiguration conf = (HibernateConfiguration) iter.next();
      conf.getHibernateService().closeSession();
    }    
  }


  public void addPlugin(ComponentPlugin plugin) {
    /*
    if(plugin instanceof AddHibernateMappingPlugin) {
      AddHibernateMappingPlugin impl = (AddHibernateMappingPlugin) plugin ;
      try {
        HibernateConfigurationImpl conf = (HibernateConfigurationImpl)getDefaultPortalConfiguration();
        conf.addMappingFiles(impl.getMapping()) ;
      } catch (Exception ex) {
        ex.printStackTrace() ;
      }
    }
    */
  }

  public ComponentPlugin removePlugin(String name) {  return null; }
  public Collection getPlugins() {    return null;  } 
}