/*
 */
/**
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Collection;

import javax.naming.CompositeName;
import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapContext;

import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Group;

/**
 * Created by The eXo Platform SARL        .
 * Author : James Chamberlain
 *          james.chamberlain@gmail.com
 */
public class ActiveDirectoryGroupHandlerImpl extends org.exoplatform.services.organization.ldap.GroupHandlerImpl {
  
  /**
   * @param ldapService
   * @param hibernateService
   */
  public ActiveDirectoryGroupHandlerImpl(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    super(ldapAttrMapping, ldapService);
  }  
  
  public void saveGroup(Group group, boolean broadcast) throws Exception {    
    LdapContext ctx = ldapService_.getLdapContext();    
    Group parent = this.findGroupById( group.getParentId());    
    setId( parent, group);             
    String groupDN = "ou=" + group.getGroupName() + ", " + this.createSubDN( parent);
    
    ArrayList<ModificationItem> modifications = new ArrayList<ModificationItem>();
    ModificationItem  mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
        new BasicAttribute( ldapAttrMapping_.ldapDescriptionAttr, group.getDescription()));
    modifications.add(mod);
    
    mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
        new BasicAttribute( ldapAttrMapping_.userDisplayNameAttr, group.getGroupName()));
    modifications.add(mod);       
    
    mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
        new BasicAttribute( "l", group.getLabel()));
    modifications.add(mod);    
    
    ModificationItem[] mods = new ModificationItem[ modifications.size()];      
    modifications.toArray(mods);
    if(broadcast) preSave( group, true);
    NameParser parser = ctx.getNameParser("");
    Name name = parser.parse( groupDN);
    ctx.modifyAttributes( name, mods);
    if(broadcast) postSave( group, true);    
  }  
}
