/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ArrayList;

import javax.faces.component.UIComponent;
import javax.faces.event.PhaseId;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Decorator;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPageForm.java,v 1.8 2004/09/29 17:44:51 benjmestrallet Exp $
 */
public class UIPageForm extends UISimpleForm implements SelectorHandler {
  static String  SEARCH_GROUP_FOR_VIEW_PERMISSION = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchGroupForViewPerm", PhaseId.APPLY_REQUEST_VALUES) ;
  static String  SEARCH_GROUP_FOR_EDIT_PERMISSION = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchGroupForEditPerm", PhaseId.APPLY_REQUEST_VALUES) ;
  
  final static String VIEW_PERMISSION = "viewPermission", EDIT_PERMISSION = "editPermission" ,
                      TITLE = "title", ICON = "icon", STATE = "state", DECORATOR = "decorator", 
                      RENDERER = "renderer", WIDTH = "width", HEIGHT = "height";
  
  private UIPage uiEditingPage_;
  private UIComponent returnComponent_ ;
  
  public UIPageForm(SkinConfigService skinService) throws Exception {
    super("pageForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    setId("UIPageForm");
    add(new UIStringInput(TITLE, "")) ;
    add(new UIStringInput(ICON, ""));
    UIExoPermissionSelector permSelector = new UIExoPermissionSelector(VIEW_PERMISSION, null) ;
    permSelector.addValidator(PermissionValidator.class);
    add(permSelector) ;
    permSelector =   new UIExoPermissionSelector(EDIT_PERMISSION, null) ;
    permSelector.addValidator(PermissionValidator.class);
    add(permSelector) ;
    add(new UIStringInput(WIDTH, "")) ;
    add(new UIStringInput(HEIGHT, "")) ;
    add(new UIStringInput(STATE, "")) ;
    add(new UISelectBox(DECORATOR, "default", new ArrayList())) ;
    UISelectBox uiSelect = 
      new UISelectBox(RENDERER, "default", Util.getRendererOptions(skinService.getPageDecorators()));
    uiSelect.setUpdateOnChangeAction("updateStyles");
    add(uiSelect) ;
    addActionListener(UpdateRendererActionListener.class, "updateStyles") ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP_FOR_VIEW_PERMISSION) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP_FOR_EDIT_PERMISSION) ;
  }
  
  public String getSearchGroupForViewPerm() { return SEARCH_GROUP_FOR_VIEW_PERMISSION; }
  public String getSearchGroupForEditPerm() { return SEARCH_GROUP_FOR_EDIT_PERMISSION; }
  
  public void setEditingPage(UIComponent returnComponent, UIPage uiPage) {
    returnComponent_ = returnComponent ;
    uiEditingPage_ = uiPage;
    Page model  = uiPage.getPageModel() ;
    String currentRenderer = model.getRenderer() ;
    if(currentRenderer == null) currentRenderer = "default" ;
    String currentDecorator = uiPage.getDecorator();
    if(currentDecorator == null || currentDecorator.length() == 0)
      currentDecorator = "default";
    
    getUIStringInput(TITLE).setValue(model.getTitle());
    getUIStringInput(ICON).setValue(model.getIcon());
    ((UIExoPermissionSelector) getUIInput(VIEW_PERMISSION)).setPermission(model.getViewPermission()) ;
    ((UIExoPermissionSelector) getUIInput(EDIT_PERMISSION)).setPermission(model.getEditPermission()) ;
    
    getUISelectBox(RENDERER).setValue(currentRenderer);
    UISelectBox uiSelectDecorator = getUISelectBox(DECORATOR) ;
    SkinConfigService skinService =  
      (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
    Decorator decorator = skinService.getPageDecorator(currentRenderer) ; 
    Util.updateSelectDecorator(uiSelectDecorator, decorator, null) ;
    
    getUIStringInput(WIDTH).setValue(model.getWidth());
    getUIStringInput(HEIGHT).setValue(model.getHeight());
    getUIStringInput(STATE).setValue(model.getState());
  }
  
  public void addNewPage() {
    reset() ;
    SkinConfigService skinService =  
      (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
    Decorator decorator = skinService.getPageDecorator("default") ; 
    UISelectBox uiSelectDecorator = getUISelectBox(DECORATOR) ;
    Util.updateSelectDecorator(uiSelectDecorator, decorator, null) ;
  }
  
  public void cancel(Selector selector) {
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }
  
  public void select(Selector selector) throws Exception {
    String info = selector.getParameters()[0];
    if(info.equals(SEARCH_GROUP_FOR_VIEW_PERMISSION)) {
      UIExoPermissionSelector permSelector =(UIExoPermissionSelector)getUIInput(VIEW_PERMISSION) ;
      permSelector.setPermission(selector.getInputValue()) ;
    } else {
      UIExoPermissionSelector permSelector = (UIExoPermissionSelector)getUIInput(EDIT_PERMISSION) ;
      permSelector.setPermission(selector.getInputValue()) ;
    }
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }
  
  static public class UpdateRendererActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageForm uiForm = (UIPageForm) event.getSource() ;
      String selectRenderer = uiForm.getUISelectBox(RENDERER).getValue() ;
      SkinConfigService skinService =  
        (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
      UISelectBox uiSelectDecorator = uiForm.getUISelectBox(DECORATOR) ;
      Decorator decorator = skinService.getPageDecorator(selectRenderer) ; 
      Util.updateSelectDecorator(uiSelectDecorator, decorator, null) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageForm uiForm = (UIPageForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
      uiPortal.setBodyComponent(uiForm.returnComponent_);
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageForm uiForm = (UIPageForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
      Page model = uiForm.uiEditingPage_.getEditablePageModel() ;
      model.setTitle(uiForm.getUIStringInput(TITLE).getValue());
      model.setIcon(uiForm.getUIStringInput(ICON).getValue());
      UIExoPermissionSelector permSelector =
        (UIExoPermissionSelector) uiForm.getUIInput(VIEW_PERMISSION) ;
      model.setViewPermission(permSelector.getPermission()) ;
      permSelector =  (UIExoPermissionSelector) uiForm.getUIInput(EDIT_PERMISSION) ;
      model.setEditPermission(permSelector.getPermission()) ;
      model.setDecorator(uiForm.getUISelectBox(DECORATOR).getValue());
      model.setRenderer(uiForm.getUISelectBox(RENDERER).getValue());
      model.setWidth(uiForm.getUIStringInput(WIDTH).getValue());
      model.setHeight(uiForm.getUIStringInput(HEIGHT).getValue());
      model.setState(uiForm.getUIStringInput(STATE).getValue());
      uiForm.uiEditingPage_.setDisplayTitle(model.getTitle()) ;
      uiForm.uiEditingPage_.updateChange() ;
      uiForm.uiEditingPage_.setComponentModified(true) ;
      uiPortal.setBodyComponent(uiForm.returnComponent_);
    }
  }
  
  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageForm uiForm = (UIPageForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser)PortalComponentCache.findPortalComponent(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm) ;
      String[] params = {event.getAction()};
      uiGroupBrowser.setParameters(params) ;
      uiPortal.setBodyComponent(uiGroupBrowser) ;
    }
  }
}