/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.Collection;
import java.util.ResourceBundle;

import org.exoplatform.commons.exception.ExoPermissionException;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.portal.PortalACL;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.Query;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.skin.SkinConfigService;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPageList.java,v 1.13 2004/10/19 13:20:02 benjmestrallet Exp $
 */
public class UIPageList extends UIGrid  {
  public static Parameter[] PREVIEW_PAGE =  { new Parameter(ACTION , "preview") } ;
  public static Parameter[] SELECT_PAGE =  { new Parameter(ACTION , "select") } ;
  public static Parameter[] DELETE_PAGE =  { new Parameter(ACTION , DELETE_ACTION) } ;
  public static Parameter[] COPY_PAGE =  { new Parameter(ACTION , "copy") } ;
  public static Parameter NEW_PAGE = new Parameter(ACTION , "newPage") ;
  public static Parameter RETURN_PAGE = new Parameter(ACTION , "return") ;
  
  public static Parameter[] newParams = { NEW_PAGE} ;
  public static Parameter[] returnParams = { RETURN_PAGE } ; 
  
  UIPageListIterator uiPageIterator_ ;
  UISearchForm uiSearchForm_ ;
  Query query_ ;
  
  public UIPageList() throws Exception {
    setId("UIPageList") ;
    setRendererType("VelocityRenderer") ;
    UIToolbar uiToolbar = new UIToolbar();
    uiToolbar.addLeftButton(new Button("#{UIPageList.button.new-page}", newParams));
    uiToolbar.addRightButton(new Button("#{UIPageList.button.cancel}", returnParams));
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    uiSearchForm_ = new UISearchForm() ;
    getChildren().add(uiPageIterator_);
    getChildren().add(uiSearchForm_);
    getChildren().add(uiToolbar);
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(SelectActionListener.class, "select") ;
    addActionListener(PreviewActionListener.class, "preview") ;
    addActionListener(CopyActionListener.class, "copy") ;
    addActionListener(NewPageActionListener.class, "newPage") ;
    addActionListener(ReturnActionListener.class, "return") ;   
  }
  
  public Collection getAllPageListInstance() { return uiPageIterator_.getPageListDataHandler().getDatas() ; } ;
  
  public Parameter[] getNewPageParams() { return newParams ; } ;
  public Parameter[] getCancelParams() { return returnParams ; } ;
  public Parameter[] getPreviewParams() { return PREVIEW_PAGE ; } ;
  public Parameter[] getSelectParams() { return SELECT_PAGE ; } ;
  public Parameter[] getDeleteParams() { return DELETE_PAGE ; } ;
  public Parameter[] getCopyParams() { return COPY_PAGE ; } ;
  
  public void queryDescriptions(String owner, String viewPerm, String editPerm) throws Exception {
    uiSearchForm_.setOwner(owner) ;
    uiSearchForm_.setViewPermission(viewPerm) ;
    uiSearchForm_.setEditPermission(editPerm) ;
    PortalConfigService service = ((UIPageBrowser) getParent()).getPortalConfigService() ;
    query_=  new Query(owner, viewPerm, editPerm) ;
    uiPageIterator_.setPageList(service.findAllPageDescriptions(query_)) ;
  }
  
  public void update() throws Exception {
    PortalConfigService service = ((UIPageBrowser) getParent()).getPortalConfigService() ;
    uiPageIterator_.setPageList(service.findAllPageDescriptions(query_)) ;
  }
 
  static public class PreviewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageList uiList = (UIPageList) event.getComponent() ;
      UIPortal uiPortal = (UIPortal) uiList.getAncestorOfType(UIPortal.class) ;
      UIPagePreview uiPagePreview = 
        (UIPagePreview)PortalComponentCache.findPortalComponent(UIPagePreview.class);
      String pageId = event.getParameter(OBJECTID) ;
      if(uiPagePreview.setPage(uiPortal, pageId)) {
        uiPortal.setBodyComponent(uiPagePreview) ;
      }
    }
  }
  
  static public class CopyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageList uiList = (UIPageList) event.getComponent() ;
      UIPortal uiPortal = (UIPortal) uiList.getAncestorOfType(UIPortal.class) ;
      String pageId = event.getParameter(OBJECTID) ;
      UIPageBrowser uiPageBrowser = (UIPageBrowser) uiList.getParent();
      PortalConfigService service = uiPageBrowser.getPortalConfigService();
      Page page = service.getPage(pageId) ;
      PortalACL acl = service.getPortalACL() ;
      if(!acl.hasViewPagePermission(page, uiPortal.getOwner())) {
        Information iprovider = findInformationProvider(uiList);
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIPageList.msg.no-view-permission"))) ;
        return ;
      }
      Page newPage = (Page)page.softCloneObject() ;
      UICopyPageForm newPageForm = new UICopyPageForm() ;
      newPageForm.setCopyPage(uiList, newPage, pageId) ;
      uiPortal.setBodyComponent(newPageForm) ;      
    }
  }
  
  static public class SelectActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIPageList uiList = (UIPageList) event.getComponent() ;
			String pageId = event.getParameter(OBJECTID);
			UIPageBrowser uiPageBrowser = (UIPageBrowser) uiList.getParent() ;
			uiPageBrowser.setSelectNodeReferencePage(pageId);
		}
	}
  
  static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIPageList uiList = (UIPageList) event.getComponent() ;
			UIPageBrowser uiPageBrowser = (UIPageBrowser) uiList.getParent();
      UIPortal uiPortal = (UIPortal) uiPageBrowser.getAncestorOfType(UIPortal.class) ;
			PortalConfigService service = uiPageBrowser.getPortalConfigService();
			String pageId = event.getParameter(OBJECTID);
      Page page = service.getPage(pageId) ;
      PortalACL acl = service.getPortalACL() ;
      if(!acl.hasEditPagePermission(page, uiPortal.getOwner())) {
        throw new ExoPermissionException(getActionName(), "onwer") ;
      }
			service.removePage(pageId);      
      PortalComponentCache  cache = 
          (PortalComponentCache) SessionContainer.getComponent(PortalComponentCache.class) ; 
			cache.removeUIPageFromCache(pageId) ;
			uiList.queryDescriptions(uiList.uiSearchForm_.getOwner(), 
                               uiList.uiSearchForm_.getViewPermission(), 
					                     uiList.uiSearchForm_.getEditPermission());
			Information ip = findInformationProvider(event.getComponent());
      ResourceBundle res = getApplicationResourceBundle() ;
			ip.addMessage(new ExoFacesMessage(res.getString("UIPageList.msg.delete-page-success")));
		}
	}
 
  static public class ReturnActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception { 
      UIPageList uiList = (UIPageList) event.getSource() ;
      UIPageBrowser uiPageBrowser = (UIPageBrowser)uiList.getParent() ;
      uiPageBrowser.goBack() ;
    }
  } 

  static public class NewPageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception { 
      UIPageList uiList = (UIPageList) event.getSource() ;
      UIPageBrowser uiPageBrowser = (UIPageBrowser)uiList.getParent() ;
      uiPageBrowser.setRenderedComponent(UIPageModelForm.class) ;
    }
  }
}