/*******************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 * Please look at license.txt in info directory for more license detail.           *
 *******************************************************************************/
package org.exoplatform.portlets.resources.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.*;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.NameValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;
import org.exoplatform.services.resources.ResourceBundleData;
import org.exoplatform.services.resources.ResourceBundleService;
/**
 * Wed, March 22, 2004 @ 23:14 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIResource.java,v 1.7 2004/08/17 13:07:01 tuan08 Exp $
 */
public class UIResource extends UISimpleForm {  

  private UITextArea textInput_ ;
  protected UISelectBox languageInput_ ;
  protected UIStringInput nameInput_ ;
  private boolean adminRole_ ;
  private ResourceBundleData data_ ;
  private ResourceBundleService service_ ;

  public UIResource(ResourceBundleService service) {
    super("resource", "post", null) ;
    setId("UIResource");    
    setClazz("UIResource") ;
    setRendererType("SimpleFormVelocityRenderer") ;
    service_ = service ;
    nameInput_ = new UIStringInput("name", "").addValidator(NameValidator.class);	
    LocaleConfigService localeService =
	      (LocaleConfigService) PortalContainer.getComponent(LocaleConfigService.class);
    Iterator i = localeService.getLocalConfigs().iterator() ;
    List options = new ArrayList() ;
    options.add(new SelectItem("All", ""))  ;
    while (i.hasNext()) {
      LocaleConfig config = (LocaleConfig) i.next() ;
      options.add(new SelectItem(config.getLocaleName(), config.getLanguage()))  ;
    }
	
    languageInput_ = new UISelectBox("language", localeService.getDefaultLocaleConfig().getLanguage(), options) ;
    languageInput_.setClazz("short") ;
    textInput_ = new UITextArea("text" , "") ;    
    textInput_.setClazz("large") ;
    adminRole_ = hasRole("admin") ;
    add(textInput_) ;
    add(languageInput_) ;
    add(nameInput_) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(PreviewActionListener.class, VIEW_ACTION) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
  
  public String getViewAction() { return VIEW_ACTION ; }
  public int getEditMode() { return EDIT_MODE ; }
  public int getViewMode() { return VIEW_MODE ; }
  public String getEditAction() { return EDIT_ACTION ; }
  public void setResourceBundleData(ResourceBundleData data) {
  	data_ = data ;
    if (data != null) {
    	textInput_.setValue(data.getData()) ;
    	languageInput_.setValue(data.getLanguage()) ;
    	languageInput_.setEditable(false) ;
    	nameInput_.setValue(data.getName()) ;
    	nameInput_.setEditable(false) ; 
    } else {
      textInput_.setValue("") ;   
    	languageInput_.setValue("") ;
    	nameInput_.setValue("") ;
    	nameInput_.setEditable(true) ;
    	languageInput_.setEditable(true) ;  
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
  		UIResource uiResource = (UIResource) event.getComponent() ;
  		UIResourcesPortlet uiPortlet = (UIResourcesPortlet)uiResource.getParent() ;
      UIListResources uiListResources = 
      	(UIListResources)uiPortlet.getChildComponentOfType(UIListResources.class);
      uiPortlet.setRenderedComponent(uiListResources.getId()) ;
    }
  }
  
  static public class EditActionListener extends ExoActionListener  {
    public EditActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
  		UIResource uiResource = (UIResource) event.getComponent() ;
  		uiResource.setMode(EDIT_MODE) ;
    }
  }
  
  static public class PreviewActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
  		UIResource uiResource = (UIResource) event.getComponent() ;
      uiResource.setMode(VIEW_MODE) ;
    }
  }
  
  static public class SaveActionListener extends ExoActionListener  {
    public SaveActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
  		UIResource uiResource = (UIResource) event.getComponent() ;
  		Information iprovider = findInformationProvider(uiResource) ;
      ResourceBundle res = getApplicationResourceBundle() ;
  		String name = uiResource.nameInput_.getValue() ;
  		if(name == null || name.length() == 0) {
  			iprovider.addMessage(new ExoFacesMessage(res.getString("UIResource.msg.resource-name-empty"))) ;
  			return ;
  		}  		
  		if(uiResource.data_ == null) {
        uiResource.data_ = uiResource.service_.createResourceBundleDataInstance() ;
        uiResource.data_.setName(uiResource.nameInput_.getValue()) ;
        uiResource.data_.setLanguage(uiResource.languageInput_.getValue()) ;
      }
      uiResource.data_.setData(uiResource.textInput_.getValue());
      uiResource.service_.saveResourceBundle(uiResource.data_);
      UIListResources uiListResources = (UIListResources)uiResource.getSibling(UIListResources.class);
      uiListResources.update() ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIResource.msg.save-resource-success"))) ;
      uiResource.reset();
    }
  }
}