/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.portal.Interceptor;
import org.exoplatform.services.portal.community.CommunityConfigService;
import org.exoplatform.services.portal.community.CommunityNavigation;
import org.exoplatform.services.portal.community.CommunityPortal;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portal.model.NodeNavigation;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.model.PageNode;
import org.exoplatform.services.portal.model.PortalConfig;
import org.hibernate.Session;
/**
 * Jul 15, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: DefaultInterceptor.java,v 1.6 2004/09/28 18:19:24 tuan08 Exp $
 */
public class DefaultPortalServiceInterceptor implements Interceptor {
  static CommunityNavigationComparator COMPARATOR = new CommunityNavigationComparator() ;
  
  private HibernateService hservice_ ;
  private CommunityConfigService communityService_ ;
  private UserConfigTemplate userConfigTemp_ ;
  
  public DefaultPortalServiceInterceptor(HibernateService hservice,
                                         CommunityConfigService communityService) throws Exception {
    hservice_ = hservice;
    communityService_ = communityService ;
    userConfigTemp_ = (UserConfigTemplate) PortalContainer.getInstance().
                getComponentInstanceOfType(UserConfigTemplate.class) ;
  }
  
  public PortalConfig getPortalConfig(String owner) throws Exception {    
    CommunityPortal cp = communityService_.findCommunityPortal(owner) ;
    String portalOwner = owner ;
    boolean shared = false ;
    if(cp != null) {
      if(!cp.getPortal().equals("#{owner}")) {        
        portalOwner = cp.getPortal() ;
        shared = true ;
      }
    }
  	PortalConfigData data = 
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, portalOwner);
  	if(data == null) {
      userConfigTemp_.createConfigurationForUser(portalOwner) ;
      data = 
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, portalOwner);
    }
    PortalConfig config = (PortalConfig)data.getPortalConfig().softCloneObject() ;
    if(shared) {         
      config.setOwner(owner) ;
      config.setEditPermission(null) ;
      config.setSharedLayout(true) ;
    } 
    return config ;
  }
  
  public void savePortalConfig(PortalConfig config) throws Exception  {
    Session session = hservice_.openSession() ;
    PortalConfigData data = 
      (PortalConfigData)session.get(PortalConfigData.class , config.getOwner()) ;
    if(data == null) {
      session.save(new PortalConfigData(config)) ;
    } else {
      data.setPortalConfig(config) ;
      session.update(data) ;
    }
    session.flush() ;
  }
  
  public Page getPage(String pageId) throws Exception  {
  	PageData data = (PageData)hservice_.findOne(PageData.class, pageId) ;
  	if(data != null)return data.getPage();
    return null ;
  }
  
  public void  savePage(Page page) throws Exception   {
     Session session =  hservice_.openSession() ;
     PageData  data =  (PageData)session.get(PageData.class,  page.getPageId()) ;
     if(data == null) {
       session.save(new PageData(page)) ;
     } else {
       data.setPage(page) ;
       session.update(data) ;
     }
     session.flush();
  }
  
  public Node getNodeNavigation(String owner) throws Exception {    
    try {
    NodeNavigationData data = 
      (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, owner) ;
    if(data == null) {
      userConfigTemp_.createConfigurationForUser(owner) ;
      data = (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, owner) ;       
    }
    PageNode pnode =  data.getNodeNavigation().getNode() ;
    NodeImpl root =  new NodeImpl(pnode, null, 0) ; 
    List list = communityService_.findCommunityNavigation(owner) ;
    Collections.sort(list, COMPARATOR) ;
    for(int i = 0 ; i < list.size() ; i++) {
      CommunityNavigation cn = (CommunityNavigation) list.get(i) ;
      data = (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, cn.getNavigation()) ;
      if(data == null) {
        communityService_.removeCommunityNavigation(cn) ;
        continue ;
      }
      PageNode sharedNode = data.getNodeNavigation().getNode();
      List children = sharedNode.getChildren() ;
      for(int j = 0 ; j < children.size() ; j++) {
        PageNode pageChild = (PageNode) children.get(j) ;
        NodeImpl shareChild = new NodeImpl(pageChild, root, 1) ;
        shareChild.setShare(true) ;
        shareChild.setGroupId(cn.getGroupId()) ;
        root.addChild(shareChild) ;
      }
    }
    
    CommunityPortal cp = communityService_.findCommunityPortal(owner) ;
    if(cp != null && cp.getUseDefaultHomePage()) {
      data = (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, cp.getPortal()) ;
      root.setPageReference(data.getNodeNavigation().getNode().getPageReference()) ;
    }
    return root ;
    } catch (Throwable t) {
      t.printStackTrace() ;
    }
    return null ;
  }
  
  public void saveNodeNavigation(String owner, Node node) throws Exception {
    NodeImpl root = (NodeImpl) node ;
    // remove shared node
    PageNode prootNode = createPageNode(root) ;
    Session session =  hservice_.openSession() ;
    NodeNavigationData data = 
      (NodeNavigationData)session.get(NodeNavigationData.class , owner) ;
    if(data == null) {
      session.save(new NodeNavigationData(owner , prootNode)) ;
    } else {
      NodeNavigation nn = new NodeNavigation() ;
      nn.setOwner(owner) ;
      nn.setNode(prootNode) ;
      data.setNodeNavigation(nn) ;
      session.update(data) ;
    }
    session.flush() ;
  }
  
  private PageNode createPageNode(NodeImpl node) {
    PageNode pnode = new PageNode() ;
    pnode.setUri(node.getUri()) ;
    pnode.setName(node.getName()) ;
    pnode.setViewPermission(node.getViewPermission()) ;
    pnode.setEditPermission(node.getEditPermission()) ;
    pnode.setIcon(node.getIcon()) ;
    pnode.setLabel(node.getLabel()) ;
    pnode.setDescription(node.getDescription()) ;
    pnode.setCreator(node.getCreator()) ;
    pnode.setModifier(node.getModifier()) ;
    pnode.setPageReference(node.getPageReference()) ;
    for(int i = 0; i < node.getChildrenSize(); i++) {
      NodeImpl child = (NodeImpl) node.getChild(i) ;
      if(!child.isShare()) {
      	pnode.getChildren().add(createPageNode(child)) ;
      }
    }
    return pnode ; 
  }
  
  static class CommunityNavigationComparator implements  Comparator {
    public int compare(Object obj1 , Object obj2) {
      CommunityNavigation cn1 = (CommunityNavigation) obj1 ;
      CommunityNavigation cn2 = (CommunityNavigation) obj2 ;
      return cn1.getPriority() - cn2.getPriority() ;
    }  
  }
}