/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.backup.ImportExportPlugin;
import org.exoplatform.services.backup.Metadata;
import org.exoplatform.services.backup.XMLObjectConverter;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.portal.model.NodeNavigation;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.model.PortalConfig;
import org.exoplatform.xml.object.XMLCollection;
import org.exoplatform.xml.object.XMLObject;
import org.hibernate.Session;
/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
public class ImportExportPluginImpl extends ImportExportPlugin {
  
  static String PLUGIN_NAME = "portal" ;
  static String META_DATA_ENTRY   = "portal/metadata.xml" ;
  static String PORTAL_CONFIG_ENTRY = "portal/portal-config.xml" ;
  static String PAGES_CONFIG_ENTRY = "portal/pages-config.xml" ;
  static String NAVIGATION_CONFIG_ENTRY = "portal/navigation-config.xml" ; 
  
  HibernateService hservice_ ;

  public ImportExportPluginImpl( HibernateService hservice, InitParams params) throws Exception {  
    hservice_ = hservice;
    init(params) ;    
  }
  
  /**
   * Return Portal ImportExportPlugin's name
   * @return Plugin's name
   */
  public String getName() {    return  PLUGIN_NAME ;}
  
  /**
   * Return description of plugin
   */
  public String getDescription() {  return "Export the user portal configuration" ; }

  /**
   * Portal ImportExportPlugin don't has service data. Then return value is always false. 
   */
  public boolean hasServiceData()  { return false ;  }

  /**
   * Export user's portal data: Portal config, portlet config, navigation config,...
   * Data export to zip file 
   * @param userName Username
   * @author os  ZipOutputStream 
   */
  public void exportUserData(String userName, ZipOutputStream os) throws Exception {
    Session session = hservice_.openSession();  
    Metadata mdata = new Metadata(getCurrentDataVersion(), getDescription(), "single-user") ;
    createEntry(META_DATA_ENTRY , os, mdata) ; 

    PortalConfigData data = (PortalConfigData) session.get( PortalConfigData.class, userName);
    if( data != null) createEntry(PORTAL_CONFIG_ENTRY, os, data.getPortalConfig()) ;   
    
    List l = 
      session.createQuery( PortalConfigServiceImpl.queryPageDataByOwner).setString(0, userName).list();
    
    List pages = new ArrayList(l.size() + 5)  ;
    for(int i = 0 ; i < l.size(); i++) {
      PageData pageData = (PageData) l.get(i) ;      
      pages.add(pageData.getPage()) ;
    }
    createEntry(PAGES_CONFIG_ENTRY, os, pages) ; 
    NodeNavigationData nvdata = 
      (NodeNavigationData) session.get(NodeNavigationData.class, userName) ;
    if( nvdata != null) createEntry(NAVIGATION_CONFIG_ENTRY, os, nvdata.getNodeNavigation()) ; 
    session.flush(); 
  }
 
  /**
   * Import user's portal data from zip file to database.
   * @param user Username (filename)
   * @param in ZipFile
   */
  public void importUserData(String user, ZipFile in) throws Exception {   
    Session session = hservice_.openSession();  
    Metadata mdata = getMetadata(META_DATA_ENTRY, in) ;
    String version = mdata.getDataVersion() ;
    XMLObjectConverter converter = getXMLObjectConverter(version) ;  
    XMLObject xmlobject;
    InputStream entry = getEntry(PORTAL_CONFIG_ENTRY, in);    
    if( entry != null){
      xmlobject = XMLObject.getXMLObject( entry);
      if( converter != null){      
        converter.traverse(xmlobject); 
        converter.pack( xmlobject);    
      }
     
      PortalConfig pconfig = (PortalConfig) xmlobject.toObject();    
      session.save( new PortalConfigData( pconfig)) ; 
    }
    entry = getEntry( PAGES_CONFIG_ENTRY, in);    
    if( entry != null){    
      XMLCollection xmlcol =  XMLCollection.getXMLCollection( entry) ;
      if(converter != null) converter.traverse( xmlcol) ;
      
      Collection pages = xmlcol.getCollection();
      Iterator i = pages.iterator() ;
      while(i.hasNext()) {
        Page page = (Page)  i.next() ;
        session.save(new PageData(page)) ;     
      }    
    }
    entry = getEntry(NAVIGATION_CONFIG_ENTRY, in);    
    if( entry != null){   
      xmlobject = XMLObject.getXMLObject(entry) ;
      if(converter != null) converter.traverse(xmlobject) ;
      NodeNavigation nn = (NodeNavigation) xmlobject.toObject() ;
      session.save(new NodeNavigationData(nn)) ;  
    }
    
    session.flush();
  }

  /**
   * Not implement for this function
   */
  public void exportServiceData( ZipOutputStream os) throws Exception {    
  }

  /**
   * Not implement for this function
   */
  public void importServiceData( ZipFile in) throws Exception {    
  }
}
