/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web.unit;

import java.util.Stack;
import org.exoplatform.test.web.ExoWebClient;
import com.meterware.httpunit.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: NewSessionUnit.java,v 1.1 2004/10/11 23:36:06 tuan08 Exp $
 **/
public class ContextManagementUnit extends WebUnit {
  static ThreadLocal tlocal_ = new ThreadLocal() {
    protected synchronized Object initialValue() {
      return new Stack() ;
    }
  };
  
  public final static int SAVE = 0 ;
  public final static int RESTORE =  1 ;
  
  private int command_  = -1;
  
  public ContextManagementUnit(int command, String description) {
    super("", description) ;
    command_ = command ;
    if(command_ ==  SAVE) setName("==>ContextManagementUnit") ;
    else if(command_ == RESTORE) setName("<==ContextManagementUnit") ;
  }
  
  public WebResponse execute(WebResponse previousResponse, WebTable block, 
  		                       ExoWebClient client) throws Exception {
    Stack stack = (Stack) tlocal_.get() ;
    if(command_ == SAVE) {
      ContextData cdata = 
        new ContextData(previousResponse,(WebConversation) client.getWebClient()) ;
      stack.push(cdata) ;
      return previousResponse ;
    } else if(command_ ==  RESTORE){
      ContextData cdata = (ContextData)stack.pop() ;
      client.setWebClient(cdata.wc, false) ;
      return cdata.previousResponse ;
    } 
    throw new Exception("Unknow command, should call saveContext()  or restoreContext") ;
  }
  
  public void log(long executionTime , int contentLength, boolean error, boolean malformed) {
    monitor_.log(0, contentLength, error, malformed) ;
  }

  public String getActionDescription() { 
    return "This unit is used to save  or restore a  browser session."; 
  }
  
  static class ContextData { 
    private WebResponse previousResponse ;
    private WebConversation wc ;
    
    ContextData(WebResponse previousResponse, WebConversation wc ) {
      this.previousResponse =  previousResponse ;
      this.wc = wc ;
    }
  }
}